/***************************************************************************
                          dialog_directories.cpp  -  description
                             -------------------
    begin                : Mon Mar 27 2000
    copyright            : (C) 2000 - 2003 by M. Hoverath
    email                : mario.hoverath@uni-koeln.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <dialog_directories.h>

/////////////////////////////////////////////////////////////////////////////
// definition of the class CDirectoriesDialog
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Constructor
CDirectoriesDialog::CDirectoriesDialog(QWidget *parent, const char *name, WFlags f)
  : CDirectoriesDialogUI(parent, name, TRUE, f)
{
  initDialog = TRUE;
  pIdList = new QPtrList<int>();
  pPluginDirectoryList = new QPtrList<QString>();
  pPluginFilterList = new QPtrList<QString>();
  if(pIdList)
  {
    pIdList->setAutoDelete(TRUE);
    pIdList->clear();
  }
  if(pPluginDirectoryList)
  {
    pPluginDirectoryList->setAutoDelete(TRUE);
    pPluginDirectoryList->clear();
  }
  if(pPluginFilterList)
  {
    pPluginFilterList->setAutoDelete(TRUE);
    pPluginFilterList->clear();
  }

  // connections
  connect(acceptButton, SIGNAL(clicked()), SLOT(accept()));
  connect(cancelButton, SIGNAL(clicked()), SLOT(reject()));
  connect(applyButton, SIGNAL(clicked()), SLOT(applyClicked()));
  connect(resetButton, SIGNAL(clicked()), SLOT(resetClicked()));
  // other signals
  connect(pluginListBox, SIGNAL(highlighted(int)), SLOT(pluginHighlighted(int)));
  connect(pluginDirLineEdit, SIGNAL(textChanged(const QString&)), SLOT(directoryChanged(const QString&)));
  connect(pluginFilterLineEdit, SIGNAL(textChanged(const QString&)), SLOT(filterChanged(const QString&)));
  connect(pluginDirLineEdit, SIGNAL(textChanged(const QString&)), SLOT(settingsChanged()));
  connect(pluginFilterLineEdit, SIGNAL(textChanged(const QString&)), SLOT(settingsChanged()));
  connect(g94ImpDirLineEdit, SIGNAL(textChanged(const QString&)), SLOT(settingsChanged()));
  connect(xyzImpDirLineEdit, SIGNAL(textChanged(const QString&)), SLOT(settingsChanged()));
  connect(xyzExpDirLineEdit, SIGNAL(textChanged(const QString&)), SLOT(settingsChanged()));
  connect(povExpDirLineEdit, SIGNAL(textChanged(const QString&)), SLOT(settingsChanged()));
  connect(vrmlExpDirLineEdit, SIGNAL(textChanged(const QString&)), SLOT(settingsChanged()));
  connect(g94ImpFilterLineEdit, SIGNAL(textChanged(const QString&)), SLOT(settingsChanged()));
  connect(xyzImpFilterLineEdit, SIGNAL(textChanged(const QString&)), SLOT(settingsChanged()));
  connect(xyzExpFilterLineEdit, SIGNAL(textChanged(const QString&)), SLOT(settingsChanged()));
  connect(povExpFilterLineEdit, SIGNAL(textChanged(const QString&)), SLOT(settingsChanged()));
  connect(vrmlExpFilterLineEdit, SIGNAL(textChanged(const QString&)), SLOT(settingsChanged()));
}

/////////////////////////////////////////////////////////////////////////////
// Destructor
CDirectoriesDialog::~CDirectoriesDialog()
{
  if(pIdList)
  {
    pIdList->clear();
    delete pIdList;
  }
  if(pPluginDirectoryList)
  {
    pPluginDirectoryList->clear();
    delete pPluginDirectoryList;
  }
  if(pPluginFilterList)
  {
    pPluginFilterList->clear();
    delete pPluginFilterList;
  }
}

/////////////////////////////////////////////////////////////////////////////
// slots
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// slot applyClicked
void CDirectoriesDialog::applyClicked(void)
{
  applyButton->setEnabled(FALSE);
}

/////////////////////////////////////////////////////////////////////////////
// slot resetClicked
void CDirectoriesDialog::resetClicked(void)
{
  int i = 0;
  kmpi::KMPI_INTERFACE_TYPE type;
  
  if(pluginManager &&
     pIdList &&
     pPluginDirectoryList &&
     pPluginFilterList)
  {
    for(int* pId = pIdList->first(); pId; pId = pIdList->next(), i++)
    {
      if((type = pluginManager->getPluginContainer(*pId)->getPlugin()->getInterfaceType()) == kmpi::KMPI_FILE_IMPORT_DIALOG ||
          type == kmpi::KMPI_FILE_EXPORT_DIALOG)
      {
        *pPluginDirectoryList->at(i) = "$HOME";
        *pPluginFilterList->at(i) = "*";
      }
      else
      {
        *pPluginDirectoryList->at(i) = "";
        *pPluginFilterList->at(i) = "";
      }
    }
  }
  
  if(pPluginDirectoryList &&
     pPluginDirectoryList->count() &&
     pPluginFilterList &&
     pPluginFilterList->count())
  {
    pluginDirLineEdit->setText(*pPluginDirectoryList->at(pluginListBox->currentItem()));
    pluginFilterLineEdit->setText(*pPluginFilterList->at(pluginListBox->currentItem()));
  }
  
  g94ImpDirLineEdit->setText("$HOME");
  xyzImpDirLineEdit->setText("$HOME");
  xyzExpDirLineEdit->setText("$HOME");
  povExpDirLineEdit->setText("$HOME");
  vrmlExpDirLineEdit->setText("$HOME");
  g94ImpFilterLineEdit->setText("*");
  xyzImpFilterLineEdit->setText("*.xyz");
  xyzExpFilterLineEdit->setText("*.xyz");
  povExpFilterLineEdit->setText("*.pov");
  vrmlExpFilterLineEdit->setText("*.wrl");
  applyButton->setEnabled(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// slot settingsChanged
void CDirectoriesDialog::settingsChanged(void)
{
  if(!initDialog) applyButton->setEnabled(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// slot pluginHighlighted
void CDirectoriesDialog::pluginHighlighted(int item)
{
  kmpi::CIPlugin* plugin = 0;
  CIPluginContainer* pluginContainer = 0;
  if(item >= 0 &&
     pluginManager &&
     pIdList &&
     pPluginDirectoryList &&
     pPluginFilterList &&
     *pIdList->at(item) >= (int)ID_PLUGIN_MIN &&
     *pIdList->at(item) <= (int)ID_PLUGIN_MAX &&
     (pluginContainer = pluginManager->getPluginContainer(*pIdList->at(item))) &&
     (plugin = pluginContainer->getPlugin()))
  {
     // show plugin properties
     pluginDirLineEdit->setText(*pPluginDirectoryList->at(item));
     pluginFilterLineEdit->setText(*pPluginFilterList->at(item));
     switch(plugin->getInterfaceType())
     {
       case kmpi::KMPI_FILE_IMPORT_DIALOG:
       case kmpi::KMPI_FILE_EXPORT_DIALOG:
         pluginDirLineEdit->setEnabled(TRUE);
         pluginFilterLineEdit->setEnabled(TRUE);
         break;
       case kmpi::KMPI_FILE_IMPORT:
       case kmpi::KMPI_FILE_EXPORT:
       default:
         pluginDirLineEdit->setEnabled(FALSE);
         pluginFilterLineEdit->setEnabled(FALSE);
         pluginDirLineEdit->setText(QString(""));
         pluginFilterLineEdit->setText(QString(""));
         break;
     }
  }
}

/////////////////////////////////////////////////////////////////////////////
// slot directoryChanged
void CDirectoriesDialog::directoryChanged(const QString& directory)
{
  if(pPluginDirectoryList) *pPluginDirectoryList->at(pluginListBox->currentItem()) = directory;
}

/////////////////////////////////////////////////////////////////////////////
// slot filterChanged
void CDirectoriesDialog::filterChanged(const QString& filter)
{
  if(pPluginFilterList) *pPluginFilterList->at(pluginListBox->currentItem()) = filter;
}

/////////////////////////////////////////////////////////////////////////////
// methods
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////
// getWorkDir
QString CDirectoriesDialog::getDirectory(int filetype)
{
  switch (filetype)
  {
    case G94IMP:
    {
      return(g94ImpDirLineEdit->text());
      break;
    }
    case XYZIMP:
    {
      return(xyzImpDirLineEdit->text());
      break;
    }
    case POVEXP:
    {
      return(povExpDirLineEdit->text());
      break;
    }
    case XYZEXP:
    {
      return(xyzExpDirLineEdit->text());
      break;
    }
    case VRMLEXP:
    {
      return(vrmlExpDirLineEdit->text());
      break;
    }
    default:
    {
      return("/");
    }
  }
}

/////////////////////////////////////////////////
// getFilter
QString CDirectoriesDialog::getFilter(int filetype)
{
  switch (filetype)
  {
    case G94IMP:
    {
      return(g94ImpFilterLineEdit->text());
      break;
    }
    case XYZIMP:
    {
      return(xyzImpFilterLineEdit->text());
      break;
    }
    case POVEXP:
    {
      return(povExpFilterLineEdit->text());
      break;
    }
    case XYZEXP:
    {
      return(xyzExpFilterLineEdit->text());
      break;
    }
    case VRMLEXP:
    {
      return(vrmlExpFilterLineEdit->text());
      break;
    }
    default:
    {
      return("*");
    }
  }
}

/////////////////////////////////////////////////
// getPluginProperties
void CDirectoriesDialog::getPluginProperties(CIPluginManager* pm)
{
  int i = 0;
  CIPluginContainer* pluginContainer = 0;
  if(pluginManager && pIdList && pPluginDirectoryList && pPluginFilterList)
  {
    for(int* pId = pIdList->first(); pId; pId = pIdList->next(), i++)
    {
      pluginContainer = pluginManager->getPluginContainer(*pId);
      pluginContainer->setWorkingDirectory(*pPluginDirectoryList->at(i));
      pluginContainer->setWorkingFilter(*pPluginFilterList->at(i));
    }
  }
  pm = pluginManager;
}

/////////////////////////////////////////////////
// setDirectory
void CDirectoriesDialog::setDirectory(QString newWorkDir, int filetype)
{
  initDialog = TRUE;
  switch (filetype)
  {
    case G94IMP:
    {
      g94ImpDirLineEdit->setText(newWorkDir);
      break;
    }
    case XYZIMP:
    {
      xyzImpDirLineEdit->setText(newWorkDir);
      break;
    }
    case POVEXP:
    {
      povExpDirLineEdit->setText(newWorkDir);
      break;
    }
    case XYZEXP:
    {
      xyzExpDirLineEdit->setText(newWorkDir);
      break;
    }
    case VRMLEXP:
    {
      vrmlExpDirLineEdit->setText(newWorkDir);
      break;
    }
    default:
    {
      newWorkDir = "/";
      g94ImpDirLineEdit->setText(newWorkDir);
      xyzImpDirLineEdit->setText(newWorkDir);
      povExpDirLineEdit->setText(newWorkDir);
      xyzExpDirLineEdit->setText(newWorkDir);
      vrmlExpDirLineEdit->setText(newWorkDir);
    }
  }
  initDialog = FALSE;
}

/////////////////////////////////////////////////
// setFilter
void CDirectoriesDialog::setFilter(QString newFilter, int filetype)
{
  initDialog = TRUE;
  switch (filetype)
  {
    case G94IMP:
    {
      g94ImpFilterLineEdit->setText(newFilter);
      break;
    }
    case XYZIMP:
    {
      xyzImpFilterLineEdit->setText(newFilter);
      break;
    }
    case POVEXP:
    {
      povExpFilterLineEdit->setText(newFilter);
      break;
    }
    case XYZEXP:
    {
      xyzExpFilterLineEdit->setText(newFilter);
      break;
    }
    case VRMLEXP:
    {
      vrmlExpFilterLineEdit->setText(newFilter);
      break;
    }
    default:
    {
      newFilter = "*";
      g94ImpFilterLineEdit->setText(newFilter);
      xyzImpFilterLineEdit->setText(newFilter);
      povExpFilterLineEdit->setText(newFilter);
      xyzExpFilterLineEdit->setText(newFilter);
      vrmlExpFilterLineEdit->setText(newFilter);
    }
  }
  initDialog = FALSE;
}

/////////////////////////////////////////////////
// setPluginProperties
void CDirectoriesDialog::setPluginProperties(CIPluginManager* pm)
{
  initDialog = TRUE;
  pluginManager = pm;
  QString newItem("");
  int i = 0;
  CIPluginContainer* pluginContainer = 0;
  if(pluginManager && pIdList && pPluginDirectoryList && pPluginFilterList)
  {
    pluginListBox->clear();
    pPluginDirectoryList->clear();
    pPluginFilterList->clear();
    for(pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_IMPORT)->first();
        pluginContainer;
        pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_IMPORT)->next())
    {
      int* id = new(int);
      *id = pluginContainer->getId();
      pIdList->insert(i++, id);
      newItem = QString(((kmpi::CIPluginFile*)pluginContainer->getPlugin())->getName().c_str());
      QPixmap pixmap(((kmpi::CIPluginFile*)pluginContainer->getPlugin())->getButtonXPM());
      pluginListBox->insertItem(pixmap, newItem);
      pPluginDirectoryList->insert(pPluginDirectoryList->count(), new QString(pluginContainer->getWorkingDirectory()));
      pPluginFilterList->insert(pPluginFilterList->count(), new QString(pluginContainer->getWorkingFilter()));
    }
    for(pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_IMPORT_DIALOG)->first();
        pluginContainer;
        pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_IMPORT_DIALOG)->next())
    {
      int* id = new(int);
      *id = pluginContainer->getId();
      pIdList->insert(i++, id);
      newItem = QString(((kmpi::CIPluginFile*)pluginContainer->getPlugin())->getName().c_str());
      QPixmap pixmap(((kmpi::CIPluginFile*)pluginContainer->getPlugin())->getButtonXPM());
      pluginListBox->insertItem(pixmap, newItem);
      pPluginDirectoryList->insert(pPluginDirectoryList->count(), new QString(pluginContainer->getWorkingDirectory()));
      pPluginFilterList->insert(pPluginFilterList->count(), new QString(pluginContainer->getWorkingFilter()));
    }
    for(pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_EXPORT)->first();
        pluginContainer;
        pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_EXPORT)->next())
    {
      int* id = new(int);
      *id = pluginContainer->getId();
      pIdList->insert(i++, id);
      newItem = QString(((kmpi::CIPluginFile*)pluginContainer->getPlugin())->getName().c_str());
      QPixmap pixmap(((kmpi::CIPluginFile*)pluginContainer->getPlugin())->getButtonXPM());
      pluginListBox->insertItem(pixmap, newItem);
      pPluginDirectoryList->insert(pPluginDirectoryList->count(), new QString(pluginContainer->getWorkingDirectory()));
      pPluginFilterList->insert(pPluginFilterList->count(), new QString(pluginContainer->getWorkingFilter()));
    }
    for(pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_EXPORT_DIALOG)->first();
        pluginContainer;
        pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_EXPORT_DIALOG)->next())
    {
      int* id = new(int);
      *id = pluginContainer->getId();
      pIdList->insert(i++, id);
      newItem = QString(((kmpi::CIPluginFile*)pluginContainer->getPlugin())->getName().c_str());
      QPixmap pixmap(((kmpi::CIPluginFile*)pluginContainer->getPlugin())->getButtonXPM());
      pluginListBox->insertItem(pixmap, newItem);
      pPluginDirectoryList->insert(pPluginDirectoryList->count(), new QString(pluginContainer->getWorkingDirectory()));
      pPluginFilterList->insert(pPluginFilterList->count(), new QString(pluginContainer->getWorkingFilter()));
    }
  }
  if(pluginManager->getNumberOfPlugins() > 0)
  {
    pluginListBox->setSelected(0, TRUE);
  }
  else
  {
    pluginDirLineEdit->setEnabled(FALSE);
    pluginFilterLineEdit->setEnabled(FALSE);
  }

  initDialog = FALSE;
}
