/***************************************************************************
                          dialog_plugin.cpp  -  description
                             -------------------
    begin                : Sun Aug 17 2003
    copyright            : (C) 2000 - 2003 by M. Hoverath
    email                : mario.hoverath@uni-koeln.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <dialog_plugin.h>

/////////////////////////////////////////////////////////////////////////////
// definition of the class CPluginDialog
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Constructor
CPluginDialog::CPluginDialog(QWidget *parent, const char *name, WFlags f)
  : CPluginDialogUI(parent, name, TRUE, f)
{
  // Initialisations
  pluginManager = 0;
  pluginListBox->clear();
  modified = FALSE;
  pluginDirectory = "/usr/lib";
  pluginFilter = "*.so*";
  pIdList = new QPtrList<int>();
  if(pIdList)
  {
    pIdList->setAutoDelete(TRUE);
    pIdList->clear();
  }
  
  // connections
  // button clicked
  connect(acceptButton, SIGNAL(clicked()), SLOT(accept()));
  connect(addPushButton, SIGNAL(clicked()), SLOT(slotAddClicked()));
  connect(deletePushButton, SIGNAL(clicked()), SLOT(slotDeleteClicked()));
  // other signals
  connect(pluginListBox, SIGNAL(highlighted(int)), SLOT(slotPluginHighlighted(int)));
}

/////////////////////////////////////////////////////////////////////////////
// Destructor
CPluginDialog::~CPluginDialog()
{
  if(pIdList)
  {
    pIdList->clear();
    delete pIdList;
  }
}

/////////////////////////////////////////////////////////////////////////////
// slotAddClicked
void CPluginDialog::slotAddClicked(void)
{
  CIPluginContainer* pluginContainer = 0;
  int nextId = - 1;
  int* pId = 0;
  int i = 0;
  bool libraryInstalled = FALSE;
  QString workingDirectory("");
  QString workingFilter("");
  QString pluginVersion("");
  QString pluginVersionMajor("");
  QString pluginVersionMinor("");
  QString pluginVersionPatch("");
  QString maxVersion(MAX_PLUGIN_VERSION);
  QString maxVersionMajor("");
  QString maxVersionMinor("");
  QString maxVersionPatch("");
  QString typeInfo("");
  // get next minimal unused id
  for(i = 0; nextId == -1 && ((int)ID_PLUGIN_MIN + i) <= (int)ID_PLUGIN_MAX && (unsigned int)i <= pluginManager->getNumberOfPlugins(); i++)
    nextId = (pluginManager->getPluginContainer((int)ID_PLUGIN_MIN + i) == 0 ? nextId = (int)ID_PLUGIN_MIN + i : -1);
  // open library
  if(nextId != -1)
  {
    // open library file
    QFileDialog fileDialog(0, 0, TRUE);
    QString library = fileDialog.getOpenFileName(pluginDirectory, pluginFilter, this);
    if(!library.isEmpty())
    {
      // check if plugin is already installed
      for(pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_IMPORT)->first();
          !libraryInstalled && pluginContainer;
          pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_IMPORT)->next())
        libraryInstalled = (pluginContainer->getLibrary() == library);
      for(pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_EXPORT)->first();
          !libraryInstalled && pluginContainer;
          pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_EXPORT)->next())
        libraryInstalled = (pluginContainer->getLibrary() == library);
      for(pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_IMPORT_DIALOG)->first();
          !libraryInstalled && pluginContainer;
          pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_IMPORT_DIALOG)->next())
        libraryInstalled = (pluginContainer->getLibrary() == library);
      for(pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_EXPORT_DIALOG)->first();
          !libraryInstalled && pluginContainer;
          pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_EXPORT_DIALOG)->next())
        libraryInstalled = (pluginContainer->getLibrary() == library);
      if(!libraryInstalled)
      {
        // add library
        modified = TRUE;
        if(pluginManager->addLibrary(library, nextId, workingDirectory, workingFilter))
        {
          if(pluginManager->getPluginContainer(nextId) &&
             pluginManager->getPluginContainer(nextId)->getPlugin() &&
             (pluginVersion = (QString(pluginManager->getPluginContainer(nextId)->getPlugin()->getInterfaceVersion().c_str()))))
          {
            // check plugin class type information
            typeInfo = QString((const char*)typeid(pluginManager->getPluginContainer(nextId)->getPlugin()).name());
            if(typeInfo.find(QString("CIPlugin"), 0, TRUE))
            {            
              // check plugin interface version
              pluginVersionMajor = pluginVersion.left(pluginVersion.find("."));
              pluginVersion = pluginVersion.right(pluginVersion.length() - pluginVersion.find(".") - 1);
              pluginVersionMinor = pluginVersion.left(pluginVersion.find("."));
              pluginVersion = pluginVersion.right(pluginVersion.length() - pluginVersion.find(".") - 1);
              pluginVersionPatch = pluginVersion.left(pluginVersion.find("."));
              maxVersionMajor = maxVersion.left(maxVersion.find("."));
              maxVersion = maxVersion.right(maxVersion.length() - maxVersion.find(".") - 1);
              maxVersionMinor = maxVersion.left(maxVersion.find("."));
              maxVersion = maxVersion.right(maxVersion.length() - maxVersion.find(".") - 1);
              maxVersionPatch = maxVersion.left(maxVersion.find("."));
              
              if((pluginVersionMajor.toInt() <= maxVersionMajor.toInt()) &&
                 (pluginVersionMinor.toInt() <= maxVersionMinor.toInt()) &&
                 (pluginVersionPatch.toInt() <= maxVersionPatch.toInt()))
              {                
                // refresh list and rewrite initfile
                if(insertPlugins())
                {
                  if(rewriteInitFile())
                  {
                    // select inserted item                    
                    for(i = 0, pId = pIdList->first(); pId; i++, pId = pIdList->next())
                      if(*pId == nextId)
                      {
                        pluginListBox->setSelected(i, TRUE);
                        break;
                      }                    
                  }
                  else
                  {
                    // rewriting initfile not successfull
                    pluginManager->deleteLibrary(nextId);
                    QMessageBox::critical(0,
                                          QString("Error"),
                                          QString("KMovisto plugin '" + library + "' is not loaded successfully.\nError while rewriting configuration to '" + initfile.GetFileName() + "'."),
                                          QMessageBox::Ok,
                                          QMessageBox::NoButton,
                                          QMessageBox::NoButton);
                  }
                }                                                                                                                                                                           
              }
              else
              {
                // interface version is too large and not supported
                pluginManager->deleteLibrary(nextId);
                QMessageBox::critical(0,
                                      QString("Error"),
                                      QString("KMovisto plugin '" + library + "' is not loaded successfully.\nSupported interface version is up to " + MAX_PLUGIN_VERSION + "."),
                                      QMessageBox::Ok,
                                      QMessageBox::NoButton,
                                      QMessageBox::NoButton);
              }
            }
            else
            {
              // interface class is not derived from CIPlugin
              pluginManager->deleteLibrary(nextId);
              QMessageBox::critical(0,
                                    QString("Error"),
                                    QString("KMovisto plugin '" + library + "' is not loaded successfully\nbecause plugin class is not derived from class 'CIPlugin'.\n(Type name is '" + typeInfo + "')."),
                                    QMessageBox::Ok,
                                    QMessageBox::NoButton,
                                    QMessageBox::NoButton);
            }           
          }
          else
          {
            // getInterfaceVersion not successfull
            pluginManager->deleteLibrary(nextId);
            QMessageBox::critical(0,
                                  QString("Error"),
                                  QString("KMovisto plugin '" + library + "' is not loaded successfully.\nUnable to resolve interface version."),
                                  QMessageBox::Ok,
                                  QMessageBox::NoButton,
                                  QMessageBox::NoButton);
          }
        }
        else
        {
          // addLibrary not successfull
          QMessageBox::critical(0,
                                QString("Error"),
                                QString("KMovisto plugin '" + library + "' is not loaded successfully.\nPlease check if the plugin library path is configured correctly.\nYou should also see the man page of 'ldconfig' for further\ninformations about installing shared libraries."),
                                QMessageBox::Ok,
                                QMessageBox::NoButton,
                                QMessageBox::NoButton);
        }
      }
      else
      {
        // addLibrary not successfull
        QMessageBox::critical(0,
                              QString("Error"),
                              QString("KMovisto plugin '" + library + "' is already installed."),
                              QMessageBox::Ok,
                              QMessageBox::NoButton,
                              QMessageBox::NoButton);
      }    
    }                
  }  
  else
  {
    QMessageBox::warning(0,
                         QString("Warning"),
                         QString("Maximum number of plugins reached. No more plugins loaded."),
                         QMessageBox::Ok,
                         QMessageBox::NoButton,
                         QMessageBox::NoButton);
  }
}

/////////////////////////////////////////////////////////////////////////////
// slotDeleteClicked
void CPluginDialog::slotDeleteClicked(void)
{
  int currentId = 0;
  int currentItem = 0;
  QString library("");
  if((currentItem = pluginListBox->currentItem()) >= 0 && (currentId = *pIdList->at(pluginListBox->currentItem())))
  {
    modified = TRUE;
    library = pluginManager->getPluginContainer(currentId)->getLibrary();
    if(pluginManager->deleteLibrary(currentId) &&
       pIdList->remove(currentItem) &&
       rewriteInitFile())
    {
      pluginListBox->removeItem(currentItem);
      if(pluginListBox->count() > 0)
      {
        pluginListBox->setSelected(0, TRUE);
      }
      else
      {
        // clear plugin properties
        descriptionTextEdit->setText("");
        fileLineEdit->setText("");
        versionLineEdit->setText("");
        typeLineEdit->setText("");
        licenseLineEdit->setText("");
        authorLineEdit->setText("");
        emailLineEdit->setText("");
        homepageLineEdit->setText("");      
      }
    }
    else
    {
      QMessageBox::critical(0,
                            QString("Error"),
                            QString("KMovisto plugin '" + library + "'is not deleted successfully."),
                            QMessageBox::Ok,
                            QMessageBox::NoButton,
                            QMessageBox::NoButton);
    }
  }
}

/////////////////////////////////////////////////////////////////////////////
// slotPluginHighlighted
void CPluginDialog::slotPluginHighlighted(int item)
{ 
  kmpi::CIPlugin* plugin = 0;
  CIPluginContainer* pluginContainer = 0;
  if(item >= 0 &&
     pluginManager &&
     pIdList &&
     *pIdList->at(item) >= (int)ID_PLUGIN_MIN &&
     *pIdList->at(item) <= (int)ID_PLUGIN_MAX &&
     (pluginContainer = pluginManager->getPluginContainer(*pIdList->at(item))) &&
     (plugin = pluginContainer->getPlugin()))
  {
     // show plugin properties
     descriptionTextEdit->setText(QString(plugin->getDescription().c_str()));     
     fileLineEdit->setText(pluginContainer->getLibrary());
     versionLineEdit->setText(QString(plugin->getPluginVersion().c_str()));
     switch(plugin->getInterfaceType())
     {
       case kmpi::KMPI_FILE_IMPORT: typeLineEdit->setText(QString("Import (without Dialog)")); break;
       case kmpi::KMPI_FILE_IMPORT_DIALOG: typeLineEdit->setText(QString("Import (with Dialog)")); break;
       case kmpi::KMPI_FILE_EXPORT: typeLineEdit->setText(QString("Export (without Dialog)")); break;
       case kmpi::KMPI_FILE_EXPORT_DIALOG: typeLineEdit->setText(QString("Export (with Dialog)")); break;
       default: typeLineEdit->setText(QString("Unknown type")); break;
     }
     licenseLineEdit->setText(QString(plugin->getLicense().c_str()));
     authorLineEdit->setText(QString(plugin->getAuthor().c_str()));
     emailLineEdit->setText(QString(plugin->getEmail().c_str()));
     homepageLineEdit->setText(QString(plugin->getHomepage().c_str()));
  }
}

/////////////////////////////////////////////////////////////////////////////
// setPluginManager
void CPluginDialog::setPluginManager(CIPluginManager* pm)
{
  pluginManager = pm;
  if(insertPlugins() && pluginManager->getNumberOfPlugins() > 0)
  {
    // if list filed set selected item to 0
    pluginListBox->setSelected(0, TRUE);
  }
}

/////////////////////////////////////////////////////////////////////////////
// getPluginManager
void CPluginDialog::getPluginManager(CIPluginManager* pm)
{
  pm = (pluginManager && pm ? pluginManager : 0);
}

/////////////////////////////////////////////////////////////////////////////
// rewriteInitFile
bool CPluginDialog::rewriteInitFile(void)
{
  CIPluginContainer* pluginContainer = 0;
  bool success = TRUE;  
  QString pluginProfile = "";
  QString pluginDirectory = "";
  QString pluginFilter = "";

  // save used profiles to pluginContainer
  for(pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_IMPORT)->first();
      pluginContainer;
      pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_IMPORT)->next())
  {
    if((pluginProfile = initfile.GetProfile("Plugin", pluginContainer->getLibrary(), "")) != "")
    {
      pluginDirectory = pluginProfile.left(pluginProfile.findRev("/", -1));
      pluginFilter = pluginProfile.right(pluginProfile.length() - pluginProfile.findRev("/", -1) - 1);
      pluginContainer->setWorkingDirectory(pluginDirectory);
      pluginContainer->setWorkingFilter(pluginFilter);
    }
  }
    
  for(pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_IMPORT_DIALOG)->first();
      pluginContainer;
      pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_IMPORT_DIALOG)->next())
  {
    if((pluginProfile = initfile.GetProfile("Plugin", pluginContainer->getLibrary(), "")) != "")
    {
      pluginDirectory = pluginProfile.left(pluginProfile.findRev("/", -1));
      pluginFilter = pluginProfile.right(pluginProfile.length() - pluginProfile.findRev("/", -1) - 1);
      pluginContainer->setWorkingDirectory(pluginDirectory);
      pluginContainer->setWorkingFilter(pluginFilter);
    }
  }

  for(pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_EXPORT)->first();
      pluginContainer;
      pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_EXPORT)->next())
  {
    if((pluginProfile = initfile.GetProfile("Plugin", pluginContainer->getLibrary(), "")) != "")
    {
      pluginDirectory = pluginProfile.left(pluginProfile.findRev("/", -1));
      pluginFilter = pluginProfile.right(pluginProfile.length() - pluginProfile.findRev("/", -1) - 1);
      pluginContainer->setWorkingDirectory(pluginDirectory);
      pluginContainer->setWorkingFilter(pluginFilter);
    }
  }

  for(pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_EXPORT_DIALOG)->first();
      pluginContainer;
      pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_EXPORT_DIALOG)->next())
  {
    if((pluginProfile = initfile.GetProfile("Plugin", pluginContainer->getLibrary(), "")) != "")
    {
      pluginDirectory = pluginProfile.left(pluginProfile.findRev("/", -1));
      pluginFilter = pluginProfile.right(pluginProfile.length() - pluginProfile.findRev("/", -1) - 1);
      pluginContainer->setWorkingDirectory(pluginDirectory);
      pluginContainer->setWorkingFilter(pluginFilter);
    }
  }

  // remove current profile
  success = (bool)initfile.RemoveProfile("Plugin");

  // rewrite profile
  for(pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_IMPORT)->first();
      pluginContainer;
      pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_IMPORT)->next())
  {  
    pluginProfile = pluginContainer->getWorkingDirectory() + "/" + pluginContainer->getWorkingFilter();
    success = (success ? success = (bool)initfile.WriteProfile("Plugin", pluginContainer->getLibrary(), pluginProfile) : FALSE);
  }

  for(pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_IMPORT_DIALOG)->first();
      pluginContainer;
      pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_IMPORT_DIALOG)->next())
  {
    pluginProfile = pluginContainer->getWorkingDirectory() + "/" + pluginContainer->getWorkingFilter();
    success = (success ? success = (bool)initfile.WriteProfile("Plugin", pluginContainer->getLibrary(), pluginProfile) : FALSE);
  }

  for(pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_EXPORT)->first();
      pluginContainer;
      pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_EXPORT)->next())
  {
    pluginProfile = pluginContainer->getWorkingDirectory() + "/" + pluginContainer->getWorkingFilter();
    success = (success ? success = (bool)initfile.WriteProfile("Plugin", pluginContainer->getLibrary(), pluginProfile) : FALSE);
  }

  for(pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_EXPORT_DIALOG)->first();
      pluginContainer;
      pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_EXPORT_DIALOG)->next())
  {
    pluginProfile = pluginContainer->getWorkingDirectory() + "/" + pluginContainer->getWorkingFilter();
    success = (success ? success = (bool)initfile.WriteProfile("Plugin", pluginContainer->getLibrary(), pluginProfile) : FALSE);
  }
  
  return(success);
}

/////////////////////////////////////////////////////////////////////////////
// rewriteInitFile
bool CPluginDialog::insertPlugins(void)
{
  QString newItem("");
  int i = 0;
  CIPluginContainer* pluginContainer = 0;
  if(pluginManager)
  {
    pluginListBox->clear();
    pIdList->clear();
    for(pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_IMPORT)->first();
        pluginContainer;
        pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_IMPORT)->next())
    {
      int* id = new(int);
      *id = pluginContainer->getId();
      pIdList->insert(i++, id);
      newItem = QString(((kmpi::CIPluginFile*)pluginContainer->getPlugin())->getName().c_str());
      QPixmap pixmap(((kmpi::CIPluginFile*)pluginContainer->getPlugin())->getButtonXPM());
      pluginListBox->insertItem(pixmap, newItem);
    }
    for(pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_IMPORT_DIALOG)->first();
        pluginContainer;
        pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_IMPORT_DIALOG)->next())
    {
      int* id = new(int);
      *id = pluginContainer->getId();
      pIdList->insert(i++, id);
      newItem = QString(((kmpi::CIPluginFile*)pluginContainer->getPlugin())->getName().c_str());
      QPixmap pixmap(((kmpi::CIPluginFile*)pluginContainer->getPlugin())->getButtonXPM());
      pluginListBox->insertItem(pixmap, newItem);
    }
    for(pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_EXPORT)->first();
        pluginContainer;
        pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_EXPORT)->next())
    {
      int* id = new(int);
      *id = pluginContainer->getId();
      pIdList->insert(i++, id);
      newItem = QString(((kmpi::CIPluginFile*)pluginContainer->getPlugin())->getName().c_str());
      QPixmap pixmap(((kmpi::CIPluginFile*)pluginContainer->getPlugin())->getButtonXPM());
      pluginListBox->insertItem(pixmap, newItem);
    }
    for(pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_EXPORT_DIALOG)->first();
        pluginContainer;
        pluginContainer = pluginManager->getPluginList(kmpi::KMPI_FILE_EXPORT_DIALOG)->next())
    {
      int* id = new(int);
      *id = pluginContainer->getId();
      pIdList->insert(i++, id);
      newItem = QString(((kmpi::CIPluginFile*)pluginContainer->getPlugin())->getName().c_str());
      QPixmap pixmap(((kmpi::CIPluginFile*)pluginContainer->getPlugin())->getButtonXPM());
      pluginListBox->insertItem(pixmap, newItem);
    }
    return(TRUE);
  }
  return(FALSE);
}



