/***************************************************************************
                          dialog_povray.h  -  description
                             -------------------
    begin                : Mon Mar 27 2000
    copyright            : (C) 2000 - 2003 by M. Hoverath
    email                : mario.hoverath@uni-koeln.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _POVRAYDIALOG
#define _POVRAYDIALOG

/* include files for QT */
#include <qvariant.h>
#include <qbuttongroup.h>
#include <qcheckbox.h>
#include <qframe.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qtabwidget.h>
#include <qwidget.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qcolordialog.h>

/* application specific includes */
#include "povfile.h"
#include "dialog_povray_ui.h"

/* class CPOVRayDialog */
class CPOVRayDialog : public CPOVRayDialogUI
{
  Q_OBJECT

public:
  /* constructor */
  CPOVRayDialog(QWidget *parent = 0, const char *name = 0, WFlags f = 0);

  /* destructor */
  ~CPOVRayDialog();

public:
  /* set ... */
  void setAlternateAtomRadius(int newAlternateAtomRadius);
  void setAlternateAtomColor(int newAlternateAtomColor);
  void setAtomColor(QColor newAtomColor);
  void setAtomRadius(float newAtomRadius);
  void setTextureNumber(int newTextureNumber);
  void setTexture(QString newTexture);
  void setInclude(QString newInclude);
  void setBondRadius(float newBondRadius);
  void setAlternateBondColor(int newAlternateBondColor);
  void setBondColor(QColor newBondColor);
  void setLight1Enabled(int newLight1Enabled);
  void setLight2Enabled(int newLight2Enabled);
  void setLight3Enabled(int newLight3Enabled);
  void setShadowless(int newShadowless);
  void setLight1(CPOVCoordinate *newLight1);
  void setLight2(CPOVCoordinate *newLight2);
  void setLight3(CPOVCoordinate *newLight3);
  void setCameraRight(CPOVCoordinate *newCameraRight);
  void setCameraUp(CPOVCoordinate *newCameraUp);
  void setCameraDirection(CPOVCoordinate *newCameraDirection);
  void setCameraLocation(CPOVCoordinate *newCameraLocation);
  void setCameraLookAt(CPOVCoordinate *newCameraLookAt);

  /* get ... */
  int getApplyStatus(void) {return(!applyButton->isEnabled());}
  int getAlternateAtomRadius(void);
  int getAlternateAtomColor(void);
  QColor getAtomColor(void);
  float getAtomRadius(void);
  int getTextureNumber(void);
  QString getTexture(void);
  QString getInclude(void);
  float getBondRadius(void);
  int getAlternateBondColor(void);
  QColor getBondColor(void);
  int getLight1Enabled(void);
  int getLight2Enabled(void);
  int getLight3Enabled(void);
  int getShadowless(void);
  CPOVCoordinate* getLight1(void);
  CPOVCoordinate* getLight2(void);
  CPOVCoordinate* getLight3(void);
  CPOVCoordinate* getCameraRight(void);
  CPOVCoordinate* getCameraUp(void);
  CPOVCoordinate* getCameraDirection(void);
  CPOVCoordinate* getCameraLocation(void);
  CPOVCoordinate* getCameraLookAt(void);

private slots:
  void applyClicked(void);
  void resetClicked(void);
  void tab1AtomColorButtonClicked(void);
  void tab1AtomTextureButtonGroupClicked(int id);
  void tab1AtomRadiusButtonGroupClicked(int id);
  void tab1AtomColorButtonGroupClicked(int id);
  void tab2BondColorButtonClicked(void);
  void tab2BondColorButtonGroupClicked(int id);
  void checkBoxChanged(int);
  void tab3light1CheckBoxChanged(int state);
  void tab3light2CheckBoxChanged(int state);
  void tab3light3CheckBoxChanged(int state);
  void lineEditChanged(const QString&);

protected:
  /* protected members */
  CPOVCoordinate* Light1;
  CPOVCoordinate* Light2;
  CPOVCoordinate* Light3;  
  CPOVCoordinate* CameraRight;
  CPOVCoordinate* CameraUp;
  CPOVCoordinate* CameraDirection;
  CPOVCoordinate* CameraLocation;
  CPOVCoordinate* CameraLookAt;
};

#endif /* _POVRAYDIALOG */
