/***************************************************************************
                          dialog_vrml.cpp  -  description
                             -------------------
    begin                : Fri Jul 14 2000
    copyright            : (C) 2000 - 2003 by M. Hoverath
    email                : mario.hoverath@uni-koeln.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <dialog_vrml.h>

/////////////////////////////////////////////////////////////////////////////
// definition of the class CVRMLDialog
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Constructor
CVRMLDialog::CVRMLDialog(QWidget *parent, const char *name, WFlags f)
  : CVRMLDialogUI(parent, name, TRUE, f)
{
  // connections
  connect(acceptButton, SIGNAL(clicked()), SLOT(accept()));
  connect(cancelButton, SIGNAL(clicked()), SLOT(reject()));
  connect(applyButton, SIGNAL(clicked()), SLOT(applyClicked()));
  connect(resetButton, SIGNAL(clicked()), SLOT(resetClicked()));
  // other signals
  connect(tab1AtomColorButton, SIGNAL(clicked()), SLOT(tab1AtomColorButtonClicked()));
  connect(tab1AtomRadiusButtonGroup, SIGNAL(clicked(int)), SLOT(tab1AtomRadiusButtonGroupClicked(int)));
  connect(tab1AtomColorButtonGroup, SIGNAL(clicked(int)), SLOT(tab1AtomColorButtonGroupClicked(int)));
  connect(tab1AtomRadiusLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab2BondColorButton, SIGNAL(clicked()), SLOT(tab2BondColorButtonClicked()));
  connect(tab2BondColorButtonGroup, SIGNAL(clicked(int)), SLOT(tab2BondColorButtonGroupClicked(int)));
  connect(tab2BondRadiusLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab3TextureButtonGroup, SIGNAL(clicked(int)), SLOT(tab3TextureButtonGroupClicked(int)));
  connect(tab3TextureURLLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
  connect(tab3DiffuseColorCheckBox, SIGNAL(stateChanged(int)), SLOT(tab3DiffuseColorCheckBoxChanged(int)));
  connect(tab3EmissiveColorCheckBox, SIGNAL(stateChanged(int)), SLOT(tab3EmissiveColorCheckBoxChanged(int)));
  connect(tab3SpecularColorCheckBox, SIGNAL(stateChanged(int)), SLOT(tab3SpecularColorCheckBoxChanged(int)));
  connect(tab3DiffuseColorSlider, SIGNAL(valueChanged(int)), SLOT(sliderChanged(int)));
  connect(tab3EmissiveColorSlider, SIGNAL(valueChanged(int)), SLOT(sliderChanged(int)));
  connect(tab3SpecularColorSlider, SIGNAL(valueChanged(int)), SLOT(sliderChanged(int)));
  connect(tab3ShininessLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
}

/////////////////////////////////////////////////////////////////////////////
// Destructor
CVRMLDialog::~CVRMLDialog()
{
}

/////////////////////////////////////////////////////////////////////////////
// slot applyClicked
void CVRMLDialog::applyClicked(void)
{
  applyButton->setEnabled(FALSE);
}

/////////////////////////////////////////////////////////////////////////////
// slot resetClicked
void CVRMLDialog::resetClicked(void)
{
  // reset tab1
  tab1AtomColor1RadioButton->setChecked(TRUE);
  tab1AtomColorButton->setEnabled(FALSE);
  tab1AtomColorFrame->setEnabled(FALSE);
  tab1AtomColorFrame->setPaletteBackgroundColor(QColor(128, 128, 128));
  tab1AtomRadius1RadioButton->setChecked(TRUE);
  tab1AtomRadiusLineEdit->setEnabled(FALSE);
  tab1AtomRadiusLineEdit->setText("");

  // reset tab2
  tab2BondColor2RadioButton->setChecked(TRUE);
  tab2BondColorButton->setEnabled(TRUE);
  tab2BondColorFrame->setEnabled(TRUE);
  tab2BondColorFrame->setPaletteBackgroundColor(QColor(204, 204, 204));
  tab2BondRadiusLineEdit->setText("0.1");

  // reset tab3
  tab3Texture1RadioButton->setChecked(TRUE);
  tab3TextureURLLineEdit->setEnabled(FALSE);
  tab3TextureURLLineEdit->setText("");
  tab3DiffuseColorCheckBox->setChecked(TRUE);
  tab3DiffuseColorSlider->setValue(100);
  tab3DiffuseColorAtomTextLabel->setEnabled(TRUE);
  tab3DiffuseColorBlackTextLabel->setEnabled(TRUE);
  tab3EmissiveColorCheckBox->setChecked(FALSE);
  tab3EmissiveColorSlider->setValue(0);
  tab3EmissiveColorAtomTextLabel->setEnabled(FALSE);
  tab3EmissiveColorBlackTextLabel->setEnabled(FALSE);
  tab3SpecularColorCheckBox->setChecked(FALSE);
  tab3SpecularColorSlider->setValue(0);
  tab3SpecularColorAtomTextLabel->setEnabled(FALSE);
  tab3SpecularColorBlackTextLabel->setEnabled(FALSE);
  tab3ShininessLineEdit->setText("1");
      
  // enable apply button
  applyButton->setEnabled(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// slot tab1AtomColorButtonClicked
void CVRMLDialog::tab1AtomColorButtonClicked(void)
{
  QColor newColor;
  newColor = QColorDialog::getColor(tab1AtomColorFrame->paletteBackgroundColor());
  if(newColor.isValid() && newColor != tab1AtomColorFrame->paletteBackgroundColor())
  {
    tab1AtomColorFrame->setPaletteBackgroundColor(newColor);
    applyButton->setEnabled(TRUE);
  }
}

/////////////////////////////////////////////////////////////////////////////
// slottab1AtomColorButtonGroupClicked
void CVRMLDialog::tab1AtomColorButtonGroupClicked(int id)
{
  switch(id)
  {
    case 1:
      tab1AtomColorButton->setEnabled(TRUE);
      tab1AtomColorFrame->setEnabled(TRUE);
      break;
    default:
      tab1AtomColorButton->setEnabled(FALSE);
      tab1AtomColorFrame->setEnabled(FALSE);
      break;
  }
  applyButton->setEnabled(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// slot tab1AtomRadiusButtonGroupClicked
void CVRMLDialog::tab1AtomRadiusButtonGroupClicked(int id)
{
  switch(id)
  {
    case 1:
      tab1AtomRadiusLineEdit->setEnabled(TRUE);
      break;
    default:
      tab1AtomRadiusLineEdit->setEnabled(FALSE);
      break;
  }
  applyButton->setEnabled(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// slot tab2BondColorButtonClicked
void CVRMLDialog::tab2BondColorButtonClicked(void)
{
  QColor newColor;
  newColor = QColorDialog::getColor(tab2BondColorFrame->paletteBackgroundColor());
  if(newColor.isValid() && newColor != tab2BondColorFrame->paletteBackgroundColor())
  {
    tab2BondColorFrame->setPaletteBackgroundColor(newColor);
    applyButton->setEnabled(TRUE);
  }
}

/////////////////////////////////////////////////////////////////////////////
// slot tab2BondColorButtonGroupClicked
void CVRMLDialog::tab2BondColorButtonGroupClicked(int id)
{
  switch(id)
  {
    case 1:
      tab2BondColorButton->setEnabled(TRUE);
      break;
    default:
      tab2BondColorButton->setEnabled(FALSE);
      break;
  }
  applyButton->setEnabled(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// slot tab3TextureButtonGroupClicked
void CVRMLDialog::tab3TextureButtonGroupClicked(int id)
{
  switch(id)
  {
    case 1:
      tab3TextureURLLineEdit->setEnabled(TRUE);
      break;
    default:
      tab3TextureURLLineEdit->setEnabled(FALSE);
      break;
  }
  applyButton->setEnabled(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// slot tab3DiffuseColorCheckBoxChanged
void CVRMLDialog::tab3DiffuseColorCheckBoxChanged(int state)
{
  if(state == 0)
  {
    tab3DiffuseColorSlider->setEnabled(FALSE);
    tab3DiffuseColorAtomTextLabel->setEnabled(FALSE);
    tab3DiffuseColorBlackTextLabel->setEnabled(FALSE);
  }
  else
  {
    tab3DiffuseColorSlider->setEnabled(TRUE);
    tab3DiffuseColorAtomTextLabel->setEnabled(TRUE);
    tab3DiffuseColorBlackTextLabel->setEnabled(TRUE);
  }
  applyButton->setEnabled(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// slot tab3EmissiveColorCheckBoxChanged
void CVRMLDialog::tab3EmissiveColorCheckBoxChanged(int state)
{
  if(state == 0)
  {
    tab3EmissiveColorSlider->setEnabled(FALSE);
    tab3EmissiveColorAtomTextLabel->setEnabled(FALSE);
    tab3EmissiveColorBlackTextLabel->setEnabled(FALSE);
  }
  else
  {
    tab3EmissiveColorSlider->setEnabled(TRUE);
    tab3EmissiveColorAtomTextLabel->setEnabled(TRUE);
    tab3EmissiveColorBlackTextLabel->setEnabled(TRUE);
  }
  applyButton->setEnabled(TRUE);
}
  
/////////////////////////////////////////////////////////////////////////////
// slot tab3SpecularColorCheckBoxChanged
void CVRMLDialog::tab3SpecularColorCheckBoxChanged(int state)
{
  if(state == 0)
  {
    tab3SpecularColorSlider->setEnabled(FALSE);
    tab3SpecularColorAtomTextLabel->setEnabled(FALSE);
    tab3SpecularColorBlackTextLabel->setEnabled(FALSE);
  }
  else
  {
    tab3SpecularColorSlider->setEnabled(TRUE);
    tab3SpecularColorAtomTextLabel->setEnabled(TRUE);
    tab3SpecularColorBlackTextLabel->setEnabled(TRUE);
  }
  applyButton->setEnabled(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// slot lineEditChanged
void CVRMLDialog::lineEditChanged(const QString&)
{
  applyButton->setEnabled(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// slot sliderChanged
void CVRMLDialog::sliderChanged(int)
{
  applyButton->setEnabled(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// methods
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////
// setAlternateAtomRadius
void CVRMLDialog::setAlternateAtomRadius(int newAlternateAtomRadius)
{
  if(newAlternateAtomRadius < 0) newAlternateAtomRadius = 0;
  if(newAlternateAtomRadius > 1) newAlternateAtomRadius = 1;
  tab1AtomRadiusButtonGroup->setButton(newAlternateAtomRadius);
  switch(newAlternateAtomRadius)
  {
    case 1:
      tab1AtomRadiusLineEdit->setEnabled(TRUE);
      break;
    default:
      tab1AtomRadiusLineEdit->setEnabled(FALSE);
      break;
  }
}

/////////////////////////////////////////////////
// setAlternateAtomColor
void CVRMLDialog::setAlternateAtomColor(int newAlternateAtomColor)
{
  if(newAlternateAtomColor < 0) newAlternateAtomColor = 0;
  if(newAlternateAtomColor > 1) newAlternateAtomColor = 1;
  tab1AtomColorButtonGroup->setButton(newAlternateAtomColor);
  switch(newAlternateAtomColor)
  {
    case 1:
      tab1AtomColorButton->setEnabled(TRUE);
      tab1AtomColorFrame->setEnabled(TRUE);
      break;
    default:
      tab1AtomColorButton->setEnabled(FALSE);
      tab1AtomColorFrame->setEnabled(FALSE);
      break;
  }
}

/////////////////////////////////////////////////
// setAtomColor
void CVRMLDialog::setAtomColor(QColor newAtomColor)
{
  tab1AtomColorFrame->setPaletteBackgroundColor(newAtomColor);
}

/////////////////////////////////////////////////
// setAtomRadius
void CVRMLDialog::setAtomRadius(float newAtomRadius)
{
  disconnect(tab1AtomRadiusLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(lineEditChanged(const QString&)));
  tab1AtomRadiusLineEdit->setText(QString::number(newAtomRadius));
  connect(tab1AtomRadiusLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
}

/////////////////////////////////////////////////
// setBondRadius
void CVRMLDialog::setBondRadius(float newBondRadius)
{
  disconnect(tab2BondRadiusLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(lineEditChanged(const QString&)));
  tab2BondRadiusLineEdit->setText(QString::number(newBondRadius));
  connect(tab2BondRadiusLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
}

/////////////////////////////////////////////////
// setAlternateBondColor
void CVRMLDialog::setAlternateBondColor(int newAlternateBondColor)
{
  if(newAlternateBondColor < 0) newAlternateBondColor = 0;
  if(newAlternateBondColor > 1) newAlternateBondColor = 1;
  tab2BondColorButtonGroup->setButton(newAlternateBondColor);
  switch(newAlternateBondColor)
  {
    case 1:
      tab2BondColorButton->setEnabled(TRUE);
      tab2BondColorFrame->setEnabled(TRUE);
      break;
    default:
      tab2BondColorButton->setEnabled(FALSE);
      tab2BondColorFrame->setEnabled(FALSE);
      break;
  } 
}

/////////////////////////////////////////////////
// setBondColor
void CVRMLDialog::setBondColor(QColor newBondColor)
{
  tab2BondColorFrame->setPaletteBackgroundColor(newBondColor);
}

/////////////////////////////////////////////////
// setEnableTextureMapping
void CVRMLDialog::setEnableTextureMapping(int newEnableTextureMapping)
{
  if(newEnableTextureMapping < 0 || newEnableTextureMapping > 1) newEnableTextureMapping = 0;
  tab3TextureButtonGroup->setButton(newEnableTextureMapping);
  switch(newEnableTextureMapping)
  {
    case 1:
      tab3TextureURLLineEdit->setEnabled(TRUE);
      break;
    default:
      tab3TextureURLLineEdit->setEnabled(FALSE);
      break;
  }
}

/////////////////////////////////////////////////
// setImageTextureURL
void CVRMLDialog::setImageTextureURL(QString newImageTextureURL)
{
  disconnect(tab3TextureURLLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(lineEditChanged(const QString&)));
  tab3TextureURLLineEdit->setText(newImageTextureURL);
  connect(tab3TextureURLLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
}

/////////////////////////////////////////////////
// setEnableDiffuseColor
void CVRMLDialog::setEnableDiffuseColor(int newEnableDiffuseColor)
{
  disconnect(tab3DiffuseColorCheckBox, SIGNAL(stateChanged(int)), this, SLOT(tab3DiffuseColorCheckBoxChanged(int)));
  tab3DiffuseColorCheckBox->setChecked((bool)newEnableDiffuseColor);
  tab3DiffuseColorSlider->setEnabled((bool)newEnableDiffuseColor);
  tab3DiffuseColorAtomTextLabel->setEnabled((bool)newEnableDiffuseColor);
  tab3DiffuseColorBlackTextLabel->setEnabled((bool)newEnableDiffuseColor);
  connect(tab3DiffuseColorCheckBox, SIGNAL(stateChanged(int)), SLOT(tab3DiffuseColorCheckBoxChanged(int)));
}

/////////////////////////////////////////////////
// setEnableEmissiveColor
void CVRMLDialog::setEnableEmissiveColor(int newEnableEmissiveColor)
{
  disconnect(tab3EmissiveColorCheckBox, SIGNAL(stateChanged(int)), this, SLOT(tab3EmissiveColorCheckBoxChanged(int)));
  tab3EmissiveColorCheckBox->setChecked((bool)newEnableEmissiveColor);
  tab3EmissiveColorSlider->setEnabled((bool)newEnableEmissiveColor);
  tab3EmissiveColorAtomTextLabel->setEnabled((bool)newEnableEmissiveColor);
  tab3EmissiveColorBlackTextLabel->setEnabled((bool)newEnableEmissiveColor);  
  connect(tab3EmissiveColorCheckBox, SIGNAL(stateChanged(int)), SLOT(tab3EmissiveColorCheckBoxChanged(int)));
}

/////////////////////////////////////////////////
// setEnableSpecularColor
void CVRMLDialog::setEnableSpecularColor(int newEnableSpecularColor)
{
  disconnect(tab3SpecularColorCheckBox, SIGNAL(stateChanged(int)), this, SLOT(tab3SpecularColorCheckBoxChanged(int)));
  tab3SpecularColorCheckBox->setChecked((bool)newEnableSpecularColor);
  tab3SpecularColorSlider->setEnabled((bool)newEnableSpecularColor);
  tab3SpecularColorAtomTextLabel->setEnabled((bool)newEnableSpecularColor);
  tab3SpecularColorBlackTextLabel->setEnabled((bool)newEnableSpecularColor);
  connect(tab3SpecularColorCheckBox, SIGNAL(stateChanged(int)), SLOT(tab3SpecularColorCheckBoxChanged(int)));
}

/////////////////////////////////////////////////
// setDiffuseColorFactor
void CVRMLDialog::setDiffuseColorFactor(float newDiffuseColorFactor)
{
  disconnect(tab3DiffuseColorSlider, SIGNAL(valueChanged(int)), this, SLOT(sliderChanged(int)));
  tab3DiffuseColorSlider->setValue(int(100 * newDiffuseColorFactor));   
  connect(tab3DiffuseColorSlider, SIGNAL(valueChanged(int)), SLOT(sliderChanged(int)));
}

/////////////////////////////////////////////////
// setEmissiveColorFactor
void CVRMLDialog::setEmissiveColorFactor(float newEmissiveColorFactor)
{
  disconnect(tab3EmissiveColorSlider, SIGNAL(valueChanged(int)), this, SLOT(sliderChanged(int)));
  tab3EmissiveColorSlider->setValue(int(100 * newEmissiveColorFactor));
  connect(tab3EmissiveColorSlider, SIGNAL(valueChanged(int)), SLOT(sliderChanged(int)));
}

/////////////////////////////////////////////////
// setSpecularColorFactor
void CVRMLDialog::setSpecularColorFactor(float newSpecularColorFactor)
{
  disconnect(tab3SpecularColorSlider, SIGNAL(valueChanged(int)), this, SLOT(sliderChanged(int)));
  tab3SpecularColorSlider->setValue(int(100 * newSpecularColorFactor));
  connect(tab3SpecularColorSlider, SIGNAL(valueChanged(int)), SLOT(sliderChanged(int)));
}

/////////////////////////////////////////////////
// setShininess
void CVRMLDialog::setShininess(float newShininess)
{
  disconnect(tab3ShininessLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(lineEditChanged(const QString&)));
  tab3ShininessLineEdit->setText(QString::number(newShininess));
  connect(tab3ShininessLineEdit, SIGNAL(textChanged(const QString&)), SLOT(lineEditChanged(const QString&)));
}

/////////////////////////////////////////////////
// getAlternateAtomRadius
int CVRMLDialog::getAlternateAtomRadius(void)
{
  return(tab1AtomRadius1RadioButton->isChecked() ? 0 : 1);
}

/////////////////////////////////////////////////
// getAlternateAtomColor
int CVRMLDialog::getAlternateAtomColor(void)
{
  return(tab1AtomColor1RadioButton->isChecked() ? 0 : 1);
}

/////////////////////////////////////////////////
// getAtomColor
QColor CVRMLDialog::getAtomColor(void)
{
  return(tab1AtomColorFrame->paletteBackgroundColor());
}

/////////////////////////////////////////////////
// getAtomRadius
float CVRMLDialog::getAtomRadius(void)
{
  return(tab1AtomRadiusLineEdit->text().toFloat() > 0.0 ? tab1AtomRadiusLineEdit->text().toFloat() : 0.5);
}

/////////////////////////////////////////////////
// getBondRadius
float CVRMLDialog::getBondRadius(void)
{
  return(tab2BondRadiusLineEdit->text().toFloat() > 0.0 ? tab2BondRadiusLineEdit->text().toFloat() : 0.1);
}

/////////////////////////////////////////////////
// getAlternateBondColor
int CVRMLDialog::getAlternateBondColor(void)
{
  return(tab2BondColor1RadioButton->isChecked() ? 0 : 1);
}

/////////////////////////////////////////////////
// getBondColor
QColor CVRMLDialog::getBondColor(void)
{
  return(tab2BondColorFrame->paletteBackgroundColor());
}

/////////////////////////////////////////////////
// getLight1Enabled
int CVRMLDialog::getEnableTextureMapping(void)
{
  return(tab3Texture2RadioButton->isChecked() ? 1 : 0);
}

/////////////////////////////////////////////////
// getLight2Enabled
QString CVRMLDialog::getImageTextureURL(void)
{
  return(tab3TextureURLLineEdit->text());
}

/////////////////////////////////////////////////
// getEnableDiffuseColor
int CVRMLDialog::getEnableDiffuseColor(void)
{
  return(tab3DiffuseColorCheckBox->isChecked() ? 1 : 0);
}

/////////////////////////////////////////////////
// getEnableEmissiveColor
int CVRMLDialog::getEnableEmissiveColor(void)
{
  return(tab3EmissiveColorCheckBox->isChecked() ? 1 : 0);
}

/////////////////////////////////////////////////
// getEnableSpecularColor
int CVRMLDialog::getEnableSpecularColor(void)
{
  return(tab3SpecularColorCheckBox->isChecked() ? 1 : 0);
}

/////////////////////////////////////////////////
// getDiffuseColorFactor
float CVRMLDialog::getDiffuseColorFactor(void)
{
  return(((float)tab3DiffuseColorSlider->value())/100);
}

/////////////////////////////////////////////////
// getEmissiveColorFactor
float CVRMLDialog::getEmissiveColorFactor(void)
{
  return(((float)tab3EmissiveColorSlider->value())/100);
}

/////////////////////////////////////////////////
// getSpecularColorFactor
float CVRMLDialog::getSpecularColorFactor(void)
{
  return(((float)tab3SpecularColorSlider->value())/100);
}

/////////////////////////////////////////////////
// getShininess
float CVRMLDialog::getShininess(void)
{
  return(tab3ShininessLineEdit->text().toFloat() >= 0.0 ? tab3ShininessLineEdit->text().toFloat() : 0.0);
}
