/***************************************************************************
                          glmolecule.h  -  description
                             -------------------
    begin                : Sat Sep 7 2002
    copyright            : (C) 2000 - 2003 by M. Hoverath
    email                : mario.hoverath@uni-koeln.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _CGLMOLECULE
#define _CGLMOLECULE

#include <qgl.h>
#include "molecule.h"

#define DEFAULT_ATOMSLICES 18
#define DEFAULT_ATOMSTACKS 9
#define DEFAULT_BONDSLICES 9
#define DEFAULT_BONDSTACKS 1
#define DEFAULT_BONDRADIUS 0.1
#define DEFAULT_MAT_EMISSION 0.4
#define DEFAULT_MAT_SPECULAR 0.5
#define DEFAULT_MAT_SHININESS 200
#define DEFAULT_LIGHT_AMBIENT 0.5
#define DEFAULT_LIGHT_SPOT 0.7
#define DEFAULT_LIGHT_POSITION_X -20.0
#define DEFAULT_LIGHT_POSITION_Y 20.0
#define DEFAULT_LIGHT_POSITION_Z 20.0
#define DEFAULT_OBJECT_DISTANCE 15.0
#define DEFAULT_MODE3D MODE3D_DISABLED
#define MODE3D_ANGLE 10.0
#define DEFAULT_GL_BGCOLOR_RED 0.0
#define DEFAULT_GL_BGCOLOR_GREEN 0.0
#define DEFAULT_GL_BGCOLOR_BLUE 0.0

#define MODE3D_DISABLED 0
#define MODE3D_COLORED_IMAGES 1
#define MODE3D_RED_GREEN 2
#define MODE3D_RED_CYAN 3
#define MODE3D_RED_BLUE 4
#define MODE3D_RED_ONLY 5
#define MODE3D_GREEN_ONLY 6
#define MODE3D_CYAN_ONLY 7
#define MODE3D_BLUE_ONLY 8

#define COLOR_WHITE 0
#define COLOR_RED 1
#define COLOR_GREEN 2
#define COLOR_CYAN 3
#define COLOR_BLUE 4

/* class CGLMolecule */
class CGLMolecule : public CMolecule
{
  Q_OBJECT

public:
  /* constructor */
  CGLMolecule(QObject *parent = 0, const char *name = 0);

  /* destructor */
  ~CGLMolecule();

  /* operators */
  CGLMolecule& operator=(CGLMolecule &mol);

public:
  /* openGL methods */
  void initializeGL(void);
  void paintGL(QGLWidget *pGLWidget);
  void resizeGL(int width, int height);
  void destroyGL(QGLWidget *pGLWidget);

  /* initialisation */
  int InitMolecule(void);

  /* Get... */
  QColor GetBackgroundColorGL(void);
  QGLFormat GetGLFormat(void);
  int GetAtomSlices(void);
  int GetAtomStacks(void);
  int GetBondSlices(void);
  int GetBondStacks(void);  
  float GetBondRadius(void);
  float GetMatEmission(void);
  float GetMatSpecular(void);
  float GetMatShininess(void);
  POINT3D GetLightAmbient(int Color);
  POINT3D GetLightSpot(int Color);
  POINT3D GetLightPosition(void);
  float GetObjectDistance(void);
  int Get3DMode(void);

  /* Set... */
  int SetBackgroundColorGL(QColor newBGColor);
  int SetAtomSlices(int newAtomSlices);
  int SetAtomStacks(int newAtomStacks);
  int SetBondSlices(int newBondSlices);
  int SetBondStacks(int newBondStacks);  
  int SetBondRadius(float newBondRadius);
  int SetMatEmission(float newMatEmission);
  int SetMatSpecular(float newMatSpecular);
  int SetMatShininess(float newMatSpecular);
  int SetLightAmbient(POINT3D newLightAmbient, int Color);
  int SetLightSpot(POINT3D newLightSpot, int Color);
  int SetLightPosition(POINT3D newLightPosition);
  int SetObjectDistance(float newObjectDistance);
  int Set3DMode(int new3DMode);

protected:
  void showElementSymbols(float radius, QString string, GLfloat redfactor = 1.0, GLfloat greenfactor = 1.0, GLfloat bluefactor = 1.0);
  void showMolecule(GLfloat redfactor = 1.0, GLfloat greenfactor = 1.0, GLfloat bluefactor = 1.0);
  void makeFontObject(void);

protected:
  GLint atomslices;
  GLint atomstacks;
  GLint bondslices;
  GLint bondstacks;
  GLfloat bondradius;
  GLfloat bgColorRedGL;
  GLfloat bgColorGreenGL;
  GLfloat bgColorBlueGL;
  GLfloat bgColorAnaglyphGL;
  GLfloat mat_emission_factor;
  GLfloat mat_specular_factor;
  GLfloat mat_shininess;
  GLfloat mat_emission[4];
  GLfloat mat_specular[4];
  GLfloat lmodel_ambient_white[4];
  GLfloat lmodel_ambient_red[4];
  GLfloat lmodel_ambient_green[4];
  GLfloat lmodel_ambient_cyan[4];
  GLfloat lmodel_ambient_blue[4];
  GLfloat light_white[4];
  GLfloat light_red[4];
  GLfloat light_green[4];
  GLfloat light_cyan[4];
  GLfloat light_blue[4];
  GLfloat light_position[4];
  GLfloat objectdistance;
  GLuint fontObject;
  int mode3D;
  QGLFormat glformat;
  GLfloat halfsize;
  static GLubyte space[];
  static GLubyte lettersUC[][13];
  static GLubyte lettersLC[][13];
  static GLubyte numbers[][13];
};

/* class CGLObject */
class CGLObject : public QGLWidget
{
  Q_OBJECT

public:
  CGLObject(QWidget* parent, const char* name);
  ~CGLObject();

  /* get... */
  int getSlices(void) {return((int)slices);};
  int getStacks(void) {return((int)stacks);};
  GLenum getDrawStyle(void) {return(drawstyle);};
  float getRadius(void) {return((float)radius);};
  QColor getColor(void) {return(color);};
  QColor getBgColor(void) {return(bgColor);};

  /* set... */
  void setSlices(int newSlices) {slices = (GLint)newSlices;};
  void setStacks(int newStacks) {stacks = (GLint)newStacks;};
  void setDrawStyle(GLenum newDrawStyle) {drawstyle = newDrawStyle;};
  void setRadius(float newRadius) {radius = (GLfloat)newRadius;};
  void setColor(QColor newColor) {color = newColor;};
  void setRotation(float newRotX, float newRotY, float newRotZ);
  void setBgColor(QColor newBgColor) {bgColor = newBgColor;};

  /* adjustments */
  void takeMoleculeProperties(const CGLMolecule* molecule);
  void putMoleculeProperties(CGLMolecule* molecule);

protected:
  void initializeGL();
  void paintGL();
  void resizeGL(int width, int height);
  virtual void drawObject(GLUquadricObj *qobj) = 0;

protected:
  QColor color;
  QColor bgColor;
  GLint slices;
  GLint stacks;
  GLenum drawstyle;
  GLfloat radius;
  GLfloat rotX;
  GLfloat rotY;
  GLfloat rotZ;
  GLfloat mat_emission_factor;
  GLfloat mat_specular_factor;
  GLfloat mat_shininess;
  GLfloat mat_emission[4];
  GLfloat mat_specular[4];
  GLfloat light_white[4];
  GLfloat lmodel_ambient_white[4];
  GLfloat light_position[4];
  GLfloat objectdistance;
};

/* class CGLAtom */
class CGLAtom : public CGLObject
{
  Q_OBJECT

public:
  CGLAtom(QWidget* parent, const char* name);
  ~CGLAtom();

  /* adjustments */
  void takeMoleculeProperties(const CGLMolecule* molecule);
  void putMoleculeProperties(CGLMolecule* molecule);

protected:
  virtual void drawObject(GLUquadricObj *qobj);
};

/* class CGLBond */  
class CGLBond : public CGLObject
{
  Q_OBJECT

public:
  CGLBond(QWidget* parent, const char* name);
  ~CGLBond();

  /* sample Atoms */
  void enableSampleAtoms(float newRadius, QColor newColor, int newAtomSlices, int newAtomStacks);
  void disableSampleAtoms(void);
    
  /* get... */
  float getLength(void) {return((float)length);};

  /* set... */
  void setLength(float newLength) {length = (GLfloat)newLength;};

  /* adjustments */
  void takeMoleculeProperties(const CGLMolecule* molecule);
  void putMoleculeProperties(CGLMolecule* molecule);

protected:
  virtual void drawObject(GLUquadricObj *qobj);

protected:
  GLfloat length;
  bool showSampleAtoms;
  GLfloat sampleAtoms_mat_emission[4];
  GLfloat sampleAtomsRadius;
  GLint sampleAtomsSlices;
  GLint sampleAtomsStacks;
};

#endif /* _CGLMOLECULE */
