/***************************************************************************
                          ipluginmanager.cpp  -  description
                             -------------------
    begin                : Mon Aug 4 2003
    copyright            : (C) 2003 by M. Hoverath
    email                : mario.hoverath@uni-koeln.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ipluginmanager.h"

/////////////////////////////////////////////////////////////////////////////
// definition of the class CIPluginContainer
/////////////////////////////////////////////////////////////////////////////

CIPluginContainer::CIPluginContainer(kmpi::CIPlugin* plugin, QString& file, void* handle, int id, QString& directory, QString& filter)
{
  pluginHandle = handle;
  pluginObject = plugin;
  idNumber = id;
  library = file;
  workingDirectory = directory;
  workingFilter = filter;
  if(pluginObject) pluginObject->onCreate();
}

CIPluginContainer::~CIPluginContainer()
{
  if(pluginObject)
  {
    pluginObject->onDestroy();
    delete pluginObject;
  }
  if(pluginHandle) dlclose((void*)pluginHandle);
}

/////////////////////////////////////////////////////////////////////////////
// definition of the class CIPluginManager
/////////////////////////////////////////////////////////////////////////////

CIPluginManager::CIPluginManager(QObject *parent, const char *name)
  : QObject(parent, name)
{
  pFileImportPluginList = new QPtrList<CIPluginContainer>();
  pFileExportPluginList = new QPtrList<CIPluginContainer>();
  pFileImportDialogPluginList = new QPtrList<CIPluginContainer>();
  pFileExportDialogPluginList = new QPtrList<CIPluginContainer>();
  if(pFileImportPluginList)
  {
    pFileImportPluginList->setAutoDelete(TRUE);
    pFileImportPluginList->clear();
  }
  if(pFileExportPluginList)
  {
    pFileExportPluginList->setAutoDelete(TRUE);
    pFileExportPluginList->clear();
  }
  if(pFileImportDialogPluginList)
  {
    pFileImportDialogPluginList->setAutoDelete(TRUE);
    pFileImportDialogPluginList->clear();
  }
  if(pFileExportDialogPluginList)
  {
    pFileExportDialogPluginList->setAutoDelete(TRUE);
    pFileExportDialogPluginList->clear();
  }
}

CIPluginManager::~CIPluginManager()
{
  if(pFileImportPluginList)
  {
    pFileImportPluginList->clear();
    delete pFileImportPluginList;
  }
  if(pFileExportPluginList)
  {
    pFileExportPluginList->clear();
    delete pFileExportPluginList;
  }
  if(pFileImportDialogPluginList)
  {
    pFileImportDialogPluginList->clear();
    delete pFileImportDialogPluginList;
  }
  if(pFileExportDialogPluginList)
  {
    pFileExportDialogPluginList->clear();
    delete pFileExportDialogPluginList;
  }
}

unsigned int CIPluginManager::getNumberOfPlugins(void)
{
  return(getNumberOfPlugins(kmpi::KMPI_FILE_IMPORT) +
         getNumberOfPlugins(kmpi::KMPI_FILE_EXPORT) +
         getNumberOfPlugins(kmpi::KMPI_FILE_IMPORT_DIALOG) +
         getNumberOfPlugins(kmpi::KMPI_FILE_EXPORT_DIALOG));
}

unsigned int CIPluginManager::getNumberOfPlugins(kmpi::KMPI_INTERFACE_TYPE type)
{
  switch(type)
  {
    case(kmpi::KMPI_FILE_IMPORT):
      return(pFileImportPluginList ? pFileImportPluginList->count() : 0);
      break;
    case(kmpi::KMPI_FILE_EXPORT):
      return(pFileExportPluginList ? pFileExportPluginList->count() : 0);
      break;
    case(kmpi::KMPI_FILE_IMPORT_DIALOG):
      return(pFileImportDialogPluginList ? pFileImportDialogPluginList->count() : 0);
      break;
    case(kmpi::KMPI_FILE_EXPORT_DIALOG):
      return(pFileExportDialogPluginList ? pFileExportDialogPluginList->count() : 0);
      break;
    default:
      return(0);
  }
}

bool CIPluginManager::addLibrary(QString& library, int id, QString& directory, QString& filter)
{
  void* pluginHandle = 0;
  kmpi::CIPlugin* pluginObject = 0;
  kmpi::CIPlugin* (*createPlugin)(void) = 0;
  CIPluginContainer* pluginContainer = 0;
  
  // open plugin library  
  if((pluginHandle = dlopen((const char*)library, RTLD_LAZY)) == 0 && dlerror() != 0) return(FALSE);
 
  // create pluginobject
  if(((createPlugin = (kmpi::CIPlugin* (*)(void))dlsym(pluginHandle, "createPlugin")) == 0 && dlerror() != 0) ||
     ((pluginObject = (*createPlugin)()) == 0))
  {
    dlclose((void*)pluginHandle);
    return(FALSE);
  }

  // create new plugin container
  if(!(pluginContainer = new CIPluginContainer(pluginObject, library, pluginHandle, id, directory, filter)))
  {
    dlclose((void*)pluginHandle);
    return(FALSE);
  }

  // add container
  return(addLibrary(pluginContainer));
}

bool CIPluginManager::addLibrary(CIPluginContainer* pluginContainer)
{
  bool insertResult = FALSE;
  if(pluginContainer && pluginContainer->getPlugin())
  {
    // add container to selected list
    switch(pluginContainer->getPlugin()->getInterfaceType())
    {
      case(kmpi::KMPI_FILE_IMPORT):
        insertResult = pFileImportPluginList->insert(pFileImportPluginList->count(), pluginContainer);
        break;
      case(kmpi::KMPI_FILE_EXPORT):
        insertResult = pFileExportPluginList->insert(pFileExportPluginList->count(), pluginContainer);
        break;
      case(kmpi::KMPI_FILE_IMPORT_DIALOG):
        insertResult = pFileImportDialogPluginList->insert(pFileImportDialogPluginList->count(), pluginContainer);
        break;
      case(kmpi::KMPI_FILE_EXPORT_DIALOG):
        insertResult = pFileExportDialogPluginList->insert(pFileExportDialogPluginList->count(), pluginContainer);
        break;
      default:
        insertResult = FALSE;
    }
  }
  return(insertResult);
}

bool CIPluginManager::deleteLibrary(int id)
{
  switch(getPlugin(id)->getInterfaceType())
  {
    case(kmpi::KMPI_FILE_IMPORT): return(pFileImportPluginList->remove(getPluginContainer(id)));
    case(kmpi::KMPI_FILE_EXPORT): return(pFileExportPluginList->remove(getPluginContainer(id)));
    case(kmpi::KMPI_FILE_IMPORT_DIALOG): return(pFileImportDialogPluginList->remove(getPluginContainer(id)));
    case(kmpi::KMPI_FILE_EXPORT_DIALOG): return(pFileExportDialogPluginList->remove(getPluginContainer(id)));
    default: return(FALSE);
  }
}

bool CIPluginManager::clear()
{
  pFileImportPluginList->clear();
  pFileExportPluginList->clear();
  pFileImportDialogPluginList->clear();
  pFileExportDialogPluginList->clear();
  return(pFileImportPluginList->count() == 0 &&
         pFileExportPluginList->count() == 0 &&
         pFileImportDialogPluginList->count() == 0 &&
         pFileExportDialogPluginList->count() == 0);  
}

bool CIPluginManager::clear(kmpi::KMPI_INTERFACE_TYPE type)
{
  switch(type)
  {
    case(kmpi::KMPI_FILE_IMPORT):
      pFileImportPluginList->clear();
      return(pFileImportPluginList->count() == 0);
      break;
    case(kmpi::KMPI_FILE_EXPORT):
      pFileExportPluginList->clear();
      return(pFileExportPluginList->count() == 0);
      break;
    case(kmpi::KMPI_FILE_IMPORT_DIALOG):
      pFileImportDialogPluginList->clear();
      return(pFileImportDialogPluginList->count() == 0);
      break;
    case(kmpi::KMPI_FILE_EXPORT_DIALOG):
      pFileExportDialogPluginList->clear();
      return(pFileExportDialogPluginList->count() == 0);
      break;
    default: break;
  }
  return(FALSE);
}

QPtrList<CIPluginContainer>* CIPluginManager::getPluginList(kmpi::KMPI_INTERFACE_TYPE type)
{
  switch(type)
  {
    case(kmpi::KMPI_FILE_IMPORT): return(pFileImportPluginList); break;
    case(kmpi::KMPI_FILE_EXPORT): return(pFileExportPluginList); break;
    case(kmpi::KMPI_FILE_IMPORT_DIALOG): return(pFileImportDialogPluginList); break;
    case(kmpi::KMPI_FILE_EXPORT_DIALOG): return(pFileExportDialogPluginList); break;
    default: return(0);   
  }
}

kmpi::CIPlugin* CIPluginManager::getPlugin(int id)
{
  CIPluginContainer* pluginContainer = 0;
  if((pluginContainer = getPluginContainer(kmpi::KMPI_FILE_IMPORT, id)) != 0) return(pluginContainer->getPlugin());
  if((pluginContainer = getPluginContainer(kmpi::KMPI_FILE_EXPORT, id)) != 0) return(pluginContainer->getPlugin());
  if((pluginContainer = getPluginContainer(kmpi::KMPI_FILE_IMPORT_DIALOG, id)) != 0) return(pluginContainer->getPlugin());
  if((pluginContainer = getPluginContainer(kmpi::KMPI_FILE_EXPORT_DIALOG, id)) != 0) return(pluginContainer->getPlugin());
  return(0);
}

CIPluginContainer* CIPluginManager::getPluginContainer(int id)
{
  CIPluginContainer* pluginContainer = 0;
  if((pluginContainer = getPluginContainer(kmpi::KMPI_FILE_IMPORT, id)) != 0) return(pluginContainer);
  if((pluginContainer = getPluginContainer(kmpi::KMPI_FILE_EXPORT, id)) != 0) return(pluginContainer);
  if((pluginContainer = getPluginContainer(kmpi::KMPI_FILE_IMPORT_DIALOG, id)) != 0) return(pluginContainer);
  if((pluginContainer = getPluginContainer(kmpi::KMPI_FILE_EXPORT_DIALOG, id)) != 0) return(pluginContainer);
  return(0);
}

CIPluginContainer* CIPluginManager::getPluginContainer(kmpi::KMPI_INTERFACE_TYPE type, int id)
{
  CIPluginContainer* pluginContainer = 0;
  switch(type)
  {
    case(kmpi::KMPI_FILE_IMPORT):
      for(pluginContainer = pFileImportPluginList->first(); pluginContainer; pluginContainer = pFileImportPluginList->next())
        if(pluginContainer->getId() == id) return(pluginContainer);
      break;
    case(kmpi::KMPI_FILE_EXPORT):
      for(pluginContainer = pFileExportPluginList->first(); pluginContainer; pluginContainer = pFileExportPluginList->next())
        if(pluginContainer->getId() == id) return(pluginContainer);
      break;
    case(kmpi::KMPI_FILE_IMPORT_DIALOG):
      for(pluginContainer = pFileImportDialogPluginList->first(); pluginContainer; pluginContainer = pFileImportDialogPluginList->next())
        if(pluginContainer->getId() == id) return(pluginContainer);
      break;
    case(kmpi::KMPI_FILE_EXPORT_DIALOG):
      for(pluginContainer = pFileExportDialogPluginList->first(); pluginContainer; pluginContainer = pFileExportDialogPluginList->next())
        if(pluginContainer->getId() == id) return(pluginContainer);
      break;
    case(kmpi::KMPI_UNKNOWN_INTERFACE):
      return(0);
      break;
  }
  return(0);
}



