/***************************************************************************
                          kmovistodoc.h  -  description
                             -------------------
    begin                : Sam Jan 22 16:27:34 CET 2000
    copyright            : (C) 2000 - 2003 by M. Hoverath
    email                : mario.hoverath@uni-koeln.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef KMOVISTODOC_H
#define KMOVISTODOC_H

// include files for QT
#include <qobject.h>
#include <qstring.h>
#include <qcolor.h>
#include <qmessagebox.h>

// application specific includes
#include "kglmolecule.h"
#include "mvtfile.h"
#include "g94file.h"
#include "xyzfile.h"
#include "povfile.h"
#include "vrmlfile.h"
#include "initfile.h"
#include "iplugin.h"

// definitions
#define G94IMP 1
#define XYZIMP 2
#define POVEXP 3
#define XYZEXP 4
#define VRMLEXP 5

/* class KMovistoDoc */
class KMovistoDoc : public QObject
{
  Q_OBJECT

public:
  KMovistoDoc();
  ~KMovistoDoc();
  void newDoc(int resetFileName = 0);
  bool save();
  bool saveAs(const QString &absfilename);
  bool load(const QString &absfilename);
  bool isModified() const;
  void setModified() {modified = TRUE;};

  // reset
  void setParametersToDefault(void);

  // get ...
  QString getMVTFileName(void) {return(MVTFile->GetFileName());};
  QString getWorkDir(int filetype);
  QString getFilter(int filetype);
  CKGLMolecule* getMolecule(void) {return(molecule);};
  CKPeriodicSystem *getPeriodicSystem(void) {return(periodicsystem);};
  float getRadiusReduction(void) {return(molecule->GetRadiusFactor());};
  float getPerspective(void) {return(molecule->GetPerspectiveFactor());};
  int getLabel(void) {return(molecule->GetLabelType());};
  float getZoomFactor(void) {return(molecule->GetZoomFactor());};
  int getAtomSlices(void) {return(molecule->GetAtomSlices());};
  int getAtomStacks(void) {return(molecule->GetAtomStacks());};
  int getBondSlices(void) {return(molecule->GetBondSlices());};
  float getBondRadius(void) {return(molecule->GetBondRadius());};  
  QColor getBondColor(void) {return(molecule->GetBondColor());};
  float getBondFudge(void) {return(molecule->GetBondFudge());};
  QString getTitle(void) {return(molecule->GetTitle());};
  QColor getBGColorPrint(void) {return(bgColorPrint);};
  QColor getBGColorGL(void) {return(molecule->GetBackgroundColorGL());};  
  QColor getBGColor(void) {return(molecule->GetBackgroundColor());};
  int getPOVAlternateAtomRadius(void) {return(AlternateAtomRadiusPOV);};
  int getPOVAlternateAtomColor(void) {return(AlternateAtomColorPOV);};
  QColor getPOVAtomColor(void) {return(AtomColorPOV);};
  float getPOVAtomRadius(void) {return(AtomRadiusPOV);};
  int getPOVTextureNumber(void) {return(TextureNumberPOV);};
  QString getPOVTexture(void) {return(TexturePOV);};
  QString getPOVInclude(void) {return(IncludePOV);};
  float getPOVBondRadius(void) {return(BondRadiusPOV);};
  int getPOVAlternateBondColor(void) {return(AlternateBondColorPOV);};
  QColor getPOVBondColor(void) {return(BondColorPOV);};
  int getPOVLight1Enabled(void) {return(Light1EnabledPOV);};
  int getPOVLight2Enabled(void) {return(Light2EnabledPOV);};
  int getPOVLight3Enabled(void) {return(Light3EnabledPOV);};
  int getPOVShadowless(void) {return(ShadowlessPOV);};
  CPOVCoordinate* getPOVLight1(void) {return(Light1POV);};
  CPOVCoordinate* getPOVLight2(void) {return(Light2POV);};
  CPOVCoordinate* getPOVLight3(void) {return(Light3POV);};
  CPOVCoordinate* getPOVCameraRight(void) {return(CameraRightPOV);};
  CPOVCoordinate* getPOVCameraUp(void) {return(CameraUpPOV);};
  CPOVCoordinate* getPOVCameraDirection(void) {return(CameraDirectionPOV);};
  CPOVCoordinate* getPOVCameraLocation(void) {return(CameraLocationPOV);};
  CPOVCoordinate* getPOVCameraLookAt(void) {return(CameraLookAtPOV);};
  int getVRMLAlternateAtomRadius(void) {return(AlternateAtomRadiusVRML);};
  int getVRMLAlternateAtomColor(void) {return(AlternateAtomColorVRML);};
  QColor getVRMLAtomColor(void) {return(AtomColorVRML);};
  float getVRMLAtomRadius(void) {return(AtomRadiusVRML);};
  float getVRMLBondRadius(void) {return(BondRadiusVRML);};
  int getVRMLAlternateBondColor(void) {return(AlternateBondColorVRML);};
  QColor getVRMLBondColor(void) {return(BondColorVRML);};
  int getVRMLEnableTextureMapping(void) {return EnableTextureMappingVRML;};
  QString getVRMLImageTextureURL(void) {return ImageTextureURLVRML;};
  int getVRMLEnableDiffuseColor(void) {return EnableDiffuseColorVRML;};
  int getVRMLEnableEmissiveColor(void) {return EnableEmissiveColorVRML;};
  int getVRMLEnableSpecularColor(void) {return EnableSpecularColorVRML;};
  float getVRMLDiffuseColorFactor(void) {return DiffuseColorFactorVRML;};
  float getVRMLEmissiveColorFactor(void) {return EmissiveColorFactorVRML;};
  float getVRMLSpecularColorFactor(void) {return SpecularColorFactorVRML;};
  float getVRMLShininess(void) {return ShininessVRML;};

  // set...
  void setMVTFileName(QString newFilename) {MVTFile->SetFileName(newFilename);};
  void setWorkDir(QString newWorkDir, int filetype);
  void setFilter(QString newFilter, int filetype);
  void setPeriodicSystem(CKPeriodicSystem *ps);
  void setMolecule(CKGLMolecule *mol);
  void setRadiusReduction(float RadiusReduction);
  void setPerspective(float Perspective);
  void setZoomFactor(float ZoomFactor);
  void setLabel(int Label);
  void setAtomSlices(int AtomSlices);
  void setAtomStacks(int AtomStacks);
  void setBondSlices(int BondSlices);
  void setBondRadius(float BondRadius);
  void setBondColor(QColor BondColor);
  void setBondFudge(float BondFudge);
  void setTitle(QString Title);
  void setBGColorPrint(QColor colorPrint);
  void setBGColorGL(QColor color);
  void setBGColor(QColor color);  
  void setPOVAlternateAtomRadius(int AlternateAtomRadius);
  void setPOVAlternateAtomColor(int AlternateAtomColor);
  void setPOVAtomColor(QColor AtomColor);
  void setPOVAtomRadius(float AtomRadius);
  void setPOVTextureNumber(int TextureNumber);
  void setPOVTexture(QString Texture);
  void setPOVInclude(QString Include);
  void setPOVBondRadius(float BondRadius);
  void setPOVAlternateBondColor(int AlternateBondColor);
  void setPOVBondColor(QColor BondColor);
  void setPOVLight1Enabled(int Light1Enabled);
  void setPOVLight2Enabled(int Light2Enabled);
  void setPOVLight3Enabled(int Light3Enabled);
  void setPOVShadowless(int Shadowless);
  void setPOVLight1(CPOVCoordinate *Light1);
  void setPOVLight2(CPOVCoordinate *Light2);
  void setPOVLight3(CPOVCoordinate *Light3);
  void setPOVCameraRight(CPOVCoordinate *CameraRight);
  void setPOVCameraUp(CPOVCoordinate *CameraUp);
  void setPOVCameraDirection(CPOVCoordinate *CameraDirection);
  void setPOVCameraLocation(CPOVCoordinate *CameraLocation);
  void setPOVCameraLookAt(CPOVCoordinate *CameraLookAt);
  void setVRMLAlternateAtomRadius(int AlternateAtomRadius);
  void setVRMLAlternateAtomColor(int AlternateAtomColor);
  void setVRMLAtomColor(QColor AtomColor);
  void setVRMLAtomRadius(float AtomRadius);
  void setVRMLBondRadius(float BondRadius);
  void setVRMLAlternateBondColor(int AlternateBondColor);
  void setVRMLBondColor(QColor BondColor);
  void setVRMLEnableTextureMapping(int EnableTextureMapping);
  void setVRMLImageTextureURL(QString ImageTextureURL);
  void setVRMLEnableDiffuseColor(int EnableDiffuseColor);
  void setVRMLEnableEmissiveColor(int EnableEmissiveColor);
  void setVRMLEnableSpecularColor(int EnableSpecularColor);
  void setVRMLDiffuseColorFactor(float DiffuseColorFactor);
  void setVRMLEmissiveColorFactor(float EmissiveColorFactor);
  void setVRMLSpecularColorFactor(float SpecularColorFactor);
  void setVRMLShininess(float Shininess);

  // import, export ...
  int importG94File(int CalNumber, CG94File &G94File);
  int importXYZFile(CXYZFile &XYZFile);
  int importPluginFile(kmpi::CIPluginMolecule &pluginMolecule, kmpi::CIPluginElements &pluginElements);  
  int exportXYZFile(CXYZFile &XYZFile);
  int exportPOVFile(CPOVFile &POVFile);
  int exportVRMLFile(CVRMLFile &VRMLFile);
  int exportPluginFile(kmpi::CIPluginMolecule &pluginMolecule, kmpi::CIPluginElements &pluginElements);

  // read initfile
  int readInitFile();
  int readMoleculeAppearanceProperties();
  int readDirectoryProperties();
  int readPOVRayProperties();
  int readVRMLProperties();
  
  // write initfile
  int writeAtomProperties(void);
  int writeBondProperties(void);
  int writeAppearanceProperties(void);
  int writePrintBGColorProperties(void);
  int writeDirectoryProperties(void);
  int writePOVRayProperties(void);
  int writeVRMLProperties(void);

signals:
  void documentChanged();

protected:
  bool modified;

  /* initfile */
  CInitFile initfile;

  /* data */
  CKGLMolecule *molecule;
  CKPeriodicSystem *periodicsystem;
  CMVTFile *MVTFile;

  /* directories */
  QString G94ImpWorkDir;
  QString XYZImpWorkDir;
  QString POVExpWorkDir;
  QString XYZExpWorkDir;
  QString VRMLExpWorkDir;

  /* filters */
  QString G94ImpFilter;
  QString XYZImpFilter;
  QString POVExpFilter;
  QString XYZExpFilter;
  QString VRMLExpFilter;

  /* background colors for printing */
  QColor bgColorPrintGL;
  QColor bgColorPrint;
  
  /* POV-Ray parameters */
  float BondRadiusPOV;
  int AlternateBondColorPOV;
  QColor BondColorPOV;
  int AlternateAtomRadiusPOV;
  int AlternateAtomColorPOV;
  QColor AtomColorPOV;
  float AtomRadiusPOV;
  int TextureNumberPOV;
  QString TexturePOV;
  QString IncludePOV;
  int Light1EnabledPOV;
  int Light2EnabledPOV;
  int Light3EnabledPOV;
  int ShadowlessPOV;
  CPOVCoordinate *Light1POV;
  CPOVCoordinate *Light2POV;
  CPOVCoordinate *Light3POV;
  CPOVCoordinate *CameraRightPOV;
  CPOVCoordinate *CameraUpPOV;
  CPOVCoordinate *CameraDirectionPOV;
  CPOVCoordinate *CameraLocationPOV;
  CPOVCoordinate *CameraLookAtPOV;

  /* VRML parameters */
  float BondRadiusVRML;
  int AlternateBondColorVRML;
  QColor BondColorVRML;
  int AlternateAtomRadiusVRML;
  int AlternateAtomColorVRML;
  QColor AtomColorVRML;
  float AtomRadiusVRML;
  int EnableTextureMappingVRML;
  QString ImageTextureURLVRML;
  int EnableDiffuseColorVRML;
  int EnableEmissiveColorVRML;
  int EnableSpecularColorVRML;
  float DiffuseColorFactorVRML;
  float EmissiveColorFactorVRML;
  float SpecularColorFactorVRML;
  float ShininessVRML;
};

#endif
