/***************************************************************************
                          povfile.h  -  description
                             -------------------
    begin                : Sun Feb 6 2000
    copyright            : (C) 2000 - 2003 by M. Hoverath
    email                : mario.hoverath@uni-koeln.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef _POVRAYFILE
#define _POVRAYFILE

/* include files for QT */
#include <qobject.h>
#include <qstring.h>
#include <qfile.h>
#include <qcolor.h>
#include <qlist.h>

/* application specific includes */
#include <iostream>

/* class CPOVCoordinate */
class CPOVCoordinate : public QObject
{
  Q_OBJECT
  friend class CPOVFile;

public:
  /* Constructor */
  CPOVCoordinate(QObject *parent=0, const char *name=0);

  /* Destructor */
  ~CPOVCoordinate();

  /* Operators */
  CPOVCoordinate& operator=(CPOVCoordinate &GPOVCoordinate);

  /* Methods */
  void Reset(void);
  void Serialize();

  /* Get... */
  float GetX(void) {return (x);}
  float GetY(void) {return (y);}
  float GetZ(void) {return (z);}

  /* Set... */
  void SetX(float newx) {x = newx; return;}
  void SetY(float newy) {y = newy; return;}
  void SetZ(float newz) {z = newz; return;}

protected:
  /* Members */
  float x;
  float y;
  float z;
};

/* class CPOVAtom */
class CPOVAtom : public QObject
{
  Q_OBJECT
  friend class CPOVFile;

public:
  /* Constructor */
  CPOVAtom(QObject *parent=0, const char *name=0);

  /* Destructor */
  ~CPOVAtom();

  /* Operators */
  CPOVAtom& operator=(CPOVAtom &CPOVAtom);

  /* Methods */
  void Serialize();

  /* Get... */
  int GetLabel(void) {return (label);}
  QColor GetColor(void) {return (color);}
  float GetRadius(void) {return (radius);}
  QString GetTexture(void) {return (texture);}
  float GetX(void) {return (atomcoordinate.GetX());}
  float GetY(void) {return (atomcoordinate.GetY());};
  float GetZ(void) {return (atomcoordinate.GetZ());}

  /* Set... */
  void SetLabel(int newlabel) {label = newlabel; return;}
  void SetColor(QColor newcolor) {color = newcolor; return;}
  void SetRadius(float newradius) {radius = newradius; return;}
  void SetTexture(QString newtexture) {texture = newtexture; return;}
  void SetX(float newx) {atomcoordinate.SetX(newx);}
  void SetY(float newy) {atomcoordinate.SetY(newy);}
  void SetZ(float newz) {atomcoordinate.SetZ(newz);}


protected:
  /* Members */
  int label;
  QColor color;
  float radius;
  QString texture;
  CPOVCoordinate atomcoordinate;
};

/* class CPOVBond */
class CPOVBond : public QObject
{
  Q_OBJECT
  friend class CPOVFile;

public:
  /* Constructor */
  CPOVBond(QObject *parent=0, const char *name=0);

  /* Destructor */
  ~CPOVBond();

  /* Operators */
  CPOVBond& operator=(CPOVBond &POVBond);

  /* Methods */
  void Serialize();

  /* Get... */
  int GetLabel(void) {return (label);}
  QColor GetColor(void) {return (color);}
  float GetRadius(void) {return (radius);}
  QString GetTexture(void) {return (texture);}
  void GetBond(CPOVCoordinate& Coordinate1, CPOVCoordinate& Coordinate2);
  float GetX(int atom);
  float GetY(int atom);
  float GetZ(int atom);

  /* Set... */
  void SetLabel(int newlabel) {label = newlabel; return;}
  void SetColor(QColor newcolor) {color = newcolor; return;}
  void SetRadius(float newradius) {radius = newradius; return;}
  void SetTexture(QString newtexture) {texture = newtexture; return;}
  void SetBond(CPOVCoordinate& Coordinate1, CPOVCoordinate& Coordinate2);
  int SetX(int atom, float newx);
  int SetY(int atom, float newy);
  int SetZ(int atom, float newz);

protected:
  /* Members */
  int label;
  QColor color;
  float radius;
  QString texture;
  CPOVCoordinate bondcoordinate[3];
};

/* class CPOVFile */
class CPOVFile : public QObject
{
  Q_OBJECT

public:
  /* constructor */
  CPOVFile(QObject *parent=0, const char *name=0);

  /* destructor */
  ~CPOVFile();

  /* Operators */
  CPOVFile& operator=(CPOVFile &POVFile);

  /* methods */
  void Reset(void);
  int SaveFile(QString Path);

  /* Get... */
  int GetNumberOfAtoms(void) {return POVAtomList.count();}  
  int GetNumberOfBonds(void) {return POVBondList.count();}
  QString GetHeader(void) {return POVHeader;};
  QString GetFoot(void) {return POVFoot;};
  QString GetTitle(void) {return POVTitle;};
  int GetShadowless(void) {return POVShadowless;};
  int GetLight1Enabled(void) {return POVLight1Enabled;};
  int GetLight2Enabled(void) {return POVLight2Enabled;};
  int GetLight3Enabled(void) {return POVLight3Enabled;};
  QString GetIncludeFile(void) {return POVIncludeFile;};
  CPOVCoordinate* GetCameraRight(void) {return POVCameraRight;};
  CPOVCoordinate* GetCameraUp(void) {return POVCameraUp;};
  CPOVCoordinate* GetCameraDirection(void) {return POVCameraDirection;};
  CPOVCoordinate* GetCameraLocation(void) {return POVCameraLocation;};
  CPOVCoordinate* GetCameraLookA(void) {return POVCameraLookAt;};
  CPOVCoordinate* GetLight1(void) {return POVLight1;};
  CPOVCoordinate* GetLight2(void) {return POVLight2;};
  CPOVCoordinate* GetLight3(void) {return POVLight3;};

  /* Set ... */
  int SetAtom(CPOVAtom* NewAtom);
  int SetBond(CPOVBond* NewBond);
  void SetHeader(QString newPOVHeader) {POVHeader = newPOVHeader; return;};
  void SetFoot(QString newPOVFoot) {POVFoot = newPOVFoot; return;};
  void SetTitle(QString newPOVTitle) {POVTitle = newPOVTitle; return;};
  void SetShadowless(int newPOVShadowless) {POVShadowless = newPOVShadowless; return;};
  void SetLight1Enabled(int newPOVLight1Enabled) {POVLight1Enabled = newPOVLight1Enabled; return;};
  void SetLight2Enabled(int newPOVLight2Enabled) {POVLight2Enabled = newPOVLight2Enabled; return;};
  void SetLight3Enabled(int newPOVLight3Enabled) {POVLight3Enabled = newPOVLight3Enabled; return;};
  void SetIncludeFile(QString newPOVIncludeFile) {POVIncludeFile = newPOVIncludeFile; return;};
  void SetCameraRight(CPOVCoordinate* newPOVCameraRight) {*POVCameraRight = *newPOVCameraRight; return;};
  void SetCameraUp(CPOVCoordinate* newPOVCameraUp) {*POVCameraUp = *newPOVCameraUp; return;};
  void SetCameraDirection(CPOVCoordinate* newPOVCameraDirection) {*POVCameraDirection = *newPOVCameraDirection; return;};
  void SetCameraLocation(CPOVCoordinate* newPOVCameraLocation) {*POVCameraLocation = *newPOVCameraLocation; return;};
  void SetCameraLookA(CPOVCoordinate* newPOVCameraLookAt) {*POVCameraLookAt = *newPOVCameraLookAt; return;};
  void SetLight1(CPOVCoordinate* newPOVLight1) {*POVLight1 = *newPOVLight1; return;};
  void SetLight2(CPOVCoordinate* newPOVLight2) {*POVLight2 = *newPOVLight2; return;};
  void SetLight3(CPOVCoordinate* newPOVLight3) {*POVLight3 = *newPOVLight3; return;};

protected:
  /* protected members */
  QList <CPOVAtom> POVAtomList;
  QList <CPOVBond> POVBondList;
  int NumberOfAtoms;
  int NumberOfBonds;
  CPOVAtom *atom;
  CPOVBond *bond;
  QString POVHeader;
  QString POVFoot;
  QString POVTitle;
  int POVShadowless;
  int POVLight1Enabled;
  int POVLight2Enabled;
  int POVLight3Enabled;
  QString POVIncludeFile;
  CPOVCoordinate* POVCameraRight;
  CPOVCoordinate* POVCameraUp;
  CPOVCoordinate* POVCameraDirection;
  CPOVCoordinate* POVCameraLocation;
  CPOVCoordinate* POVCameraLookAt;
  CPOVCoordinate* POVLight1;
  CPOVCoordinate* POVLight2;
  CPOVCoordinate* POVLight3;
};

#endif /* _POVRAYFILE */
