/***************************************************************************
                          vrmlfile.cpp  -  description
                             -------------------
    begin                : Sun Jul 9 2000
    copyright            : (C) 2000 - 2003 by M. Hoverath
    email                : mario.hoverath@uni-koeln.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files
#include <vrmlfile.h>

/////////////////////////////////////////////////////////////////////////////
// definition of the class CVRMLCoordinate
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Constructor
CVRMLCoordinate::CVRMLCoordinate(QObject *parent, const char *name)
  : QObject(parent, name)
{
  Reset();
}

/////////////////////////////////////////////////////////////////////////////
// destructor
CVRMLCoordinate::~CVRMLCoordinate()
{

}

/////////////////////////////////////////////////////////////////////////////
// Reset
void CVRMLCoordinate::Reset(void)
{
  x = (float)0.0;
  y = (float)0.0;
  z = (float)0.0;
}

/////////////////////////////////////////////////////////////////////////////
// = operator
CVRMLCoordinate& CVRMLCoordinate::operator=(CVRMLCoordinate &VRMLCoordinate)
{
  // equalize members
  x = VRMLCoordinate.x;
  y = VRMLCoordinate.y;
  z = VRMLCoordinate.z;

  return *this;
}

/////////////////////////////////////////////////////////////////////////////
// Serialize
void CVRMLCoordinate::Serialize()
{
  // not yet implemented
}

/////////////////////////////////////////////////////////////////////////////
// definition of the class CVRMLAtom
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Constructor
CVRMLAtom::CVRMLAtom(QObject *parent, const char *name)
  : QObject(parent, name)
{
  label = 0;
  color.setRgb(0, 0, 0);
  radius = (float)0.0;
  atomcoordinate.Reset();
}

/////////////////////////////////////////////////////////////////////////////
// destructor
CVRMLAtom::~CVRMLAtom()
{

}

/////////////////////////////////////////////////////////////////////////////
// = operator
CVRMLAtom& CVRMLAtom::operator=(CVRMLAtom &VRMLAtom)
{
  // equalize members
  label = VRMLAtom.label;
  color = VRMLAtom.color;
  radius = VRMLAtom.radius;
  atomcoordinate = VRMLAtom.atomcoordinate;
  return *this;
}

/////////////////////////////////////////////////////////////////////////////
// Serialize
void CVRMLAtom::Serialize()
{
  // not yet implemented
}

/////////////////////////////////////////////////////////////////////////////
// definition of the class CVRMLBond
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Constructor
CVRMLBond::CVRMLBond(QObject *parent, const char *name)
  : QObject(parent, name)
{
  label = 0;
  color.setRgb(0, 0, 0);
  radius = (float)0.0;
  bondcoordinate[1].Reset();
  bondcoordinate[2].Reset();
}

/////////////////////////////////////////////////////////////////////////////
// destructor
CVRMLBond::~CVRMLBond()
{

}

/////////////////////////////////////////////////////////////////////////////
// = operator
CVRMLBond& CVRMLBond::operator=(CVRMLBond &VRMLBond)
{
  // equalize members
  label = VRMLBond.label;
  color = VRMLBond.color;
  radius = VRMLBond.radius;
  bondcoordinate[1] = VRMLBond.bondcoordinate[1];
  bondcoordinate[2] = VRMLBond.bondcoordinate[2];

  return *this;
}

/////////////////////////////////////////////////////////////////////////////
// Serialize
void CVRMLBond::Serialize()
{
  // not yet implemented
}


/////////////////////////////////////////////////////////////////////////////
// GetX
float CVRMLBond::GetX(int atom)
{
  if(atom != 1 && atom != 2) return 0.0;
  return bondcoordinate[atom].GetX();
}

/////////////////////////////////////////////////////////////////////////////
// GetY
float CVRMLBond::GetY(int atom)
{
  if(atom != 1 && atom != 2) return 0.0;
  return bondcoordinate[atom].GetY();
}

/////////////////////////////////////////////////////////////////////////////
// GetZ
float CVRMLBond::GetZ(int atom)
{
  if(atom != 1 && atom != 2) return 0.0;
  return bondcoordinate[atom].GetZ();
}

/////////////////////////////////////////////////////////////////////////////
// GetBond
void CVRMLBond::GetBond(CVRMLCoordinate& Coordinate1, CVRMLCoordinate& Coordinate2)
{
  Coordinate1 = bondcoordinate[1];
  Coordinate2 = bondcoordinate[2];
}

/////////////////////////////////////////////////////////////////////////////
// SetX
int CVRMLBond::SetX(int atom, float newx)
{
  if(atom != 1 && atom != 2) return FALSE;
  bondcoordinate[atom].SetX(newx);
  return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// SetY
int CVRMLBond::SetY(int atom, float newy)
{
  if(atom != 1 && atom != 2) return FALSE;
  bondcoordinate[atom].SetY(newy);
  return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// SetZ
int CVRMLBond::SetZ(int atom, float newz)
{
  if(atom != 1 && atom != 2) return FALSE;
  bondcoordinate[atom].SetZ(newz);
  return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// SetBond
void CVRMLBond::SetBond(CVRMLCoordinate& Coordinate1, CVRMLCoordinate& Coordinate2)
{
  bondcoordinate[1] = Coordinate1;
  bondcoordinate[2] = Coordinate2;
}

/////////////////////////////////////////////////////////////////////////////
// definition of the class CVRMLFile
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Constructor
CVRMLFile::CVRMLFile(QObject *parent, const char *name)
  : QObject(parent, name)
{
  // initialisations
  // enabling auto-delete will delete objects that are removed
  VRMLAtomList.setAutoDelete(TRUE);
  VRMLBondList.setAutoDelete(TRUE);
  // reset members;
  Reset();
}

/////////////////////////////////////////////////////////////////////////////
// destructor
CVRMLFile::~CVRMLFile()
{
  // auto-delete is enabled!
  VRMLAtomList.clear();
  VRMLBondList.clear();
}

/////////////////////////////////////////////////////////////////////////////
// = operator
CVRMLFile& CVRMLFile::operator=(CVRMLFile &VRMLFile)
{
  // clear
  Reset();
  // copy members
  VRMLHeader = VRMLFile.VRMLHeader;
  VRMLFoot = VRMLFile.VRMLFoot;
  VRMLTitle = VRMLFile.VRMLTitle;
  VRMLEnableTextureMapping = VRMLFile.VRMLEnableTextureMapping;
  VRMLImageTextureURL = VRMLFile.VRMLImageTextureURL;
  VRMLEnableDiffuseColor = VRMLFile.VRMLEnableDiffuseColor;
  VRMLEnableEmissiveColor = VRMLFile.VRMLEnableEmissiveColor;
  VRMLEnableSpecularColor = VRMLFile.VRMLEnableSpecularColor;
  VRMLDiffuseColorFactor = VRMLFile.VRMLDiffuseColorFactor;
  VRMLEmissiveColorFactor = VRMLFile.VRMLEmissiveColorFactor;
  VRMLSpecularColorFactor = VRMLFile.VRMLSpecularColorFactor;
  VRMLShininess = VRMLFile.VRMLShininess;

  // VRMLAtomcoordinate-List
  VRMLAtomList.clear();
  CVRMLAtom atomBuffer;
  if(VRMLFile.VRMLAtomList.first())
  {
    for(uint i = 0; i <= VRMLFile.VRMLAtomList.count() - 1; i++)
    {
      atomBuffer = *(CVRMLAtom*)VRMLFile.VRMLAtomList.current();
      atom = new CVRMLAtom;
      if(!atom)
      {
        Reset();
        return *this;
      }
      *atom = atomBuffer;
      VRMLAtomList.append(atom);
      VRMLFile.VRMLAtomList.next();
    }
  }
  // VRMLAtombond-List
  VRMLBondList.clear();
  CVRMLBond bondBuffer;
  if(VRMLFile.VRMLBondList.first())
  {
    for(uint i = 0; i <= VRMLFile.VRMLBondList.count() - 1; i++)
    {
      bondBuffer = *(CVRMLBond*)VRMLFile.VRMLBondList.current();
      bond = new CVRMLBond;
      if(!bond)
      {
        Reset();
        return *this;
      }
      *bond = bondBuffer;
      VRMLBondList.append(bond);
      VRMLFile.VRMLBondList.next();
    }
  }
  return *this;
}

/////////////////////////////////////////////////////////////////////////////
// Reset
void CVRMLFile::Reset(void)
{
  // reset lists
  VRMLAtomList.clear();
  VRMLBondList.clear();
  // reset members;
  NumberOfAtoms = 0;
  NumberOfBonds = 0;
  VRMLHeader = "";
  VRMLFoot = "";
  VRMLTitle = "";
  VRMLEnableTextureMapping = 0;
  VRMLImageTextureURL = "";
  VRMLEnableDiffuseColor = 1;
  VRMLEnableEmissiveColor = 0;
  VRMLEnableSpecularColor = 0;
  VRMLDiffuseColorFactor = 1.0;
  VRMLEmissiveColorFactor = 0.0;
  VRMLSpecularColorFactor = 0.0;
  VRMLShininess = 0.0;
}

/////////////////////////////////////////////////////////////////////////////
// SaveFile
int CVRMLFile::SaveFile(QString Path)
{
  // Initialisations
  CVRMLAtom* atomBuffer;
  CVRMLBond* bondBuffer;
  QString LineBuffer = "";
  int i = 0;
  QFile VRMLFile;
  QString Line = "";
  float Atom1x = 0.0;
  float Atom1y = 0.0;
  float Atom1z = 0.0;
  float Atom2x = 0.0;
  float Atom2y = 0.0;
  float Atom2z = 0.0;
  float Bondx = 0.0;
  float Bondy = 0.0;
  float Bondz = 0.0;
  float BondLength = 0.0;
  // try to open VRML-file in write-only mode
  VRMLFile.setName(Path);
  if (VRMLFile.open(IO_WriteOnly))
  {
    // write first line
    Line = "#VRML V2.0 utf8\r\n";
    VRMLFile.writeBlock(Line, Line.length());
    // write header
    if(VRMLHeader != "")
    {
      Line = "# " + VRMLHeader + "\r\n\r\n";
      VRMLFile.writeBlock(Line, Line.length());
    }
    // write Title:
    if(VRMLTitle != "")
    {
      Line = "# Title: " + VRMLTitle + "\r\n\r\n";
      VRMLFile.writeBlock(Line, Line.length());
    }
    // write white background
    Line = "Background { skyColor 1.0 1.0 1.0 }\r\n\r\n";
    VRMLFile.writeBlock(Line, Line.length());
    // write atoms
    if(NumberOfAtoms > 0)
    {
      Line = "# ATOMS\r\n";
      VRMLFile.writeBlock(Line, Line.length());
      VRMLAtomList.first();
      for(i = 1; i <= NumberOfAtoms; i++)
      {
        if(!(atomBuffer = (CVRMLAtom*)VRMLAtomList.current())) return FALSE;
        VRMLAtomList.next();
        Line = "Transform {\r\n  children [\r\n    Shape {\r\n      appearance Appearance {\r\n";
        // texture mapping
        if(VRMLEnableTextureMapping)
        {
          Line += "        texture ImageTexture {\r\n";
          Line += "          url ";
          if (VRMLImageTextureURL.left(1) != "\"") Line += "\"";
          Line += VRMLImageTextureURL;
          if (VRMLImageTextureURL.right(1) != "\"") Line += "\"";
          Line += "\r\n        }\r\n";
        }
        // material
        Line += "        material Material {\r\n";
        VRMLFile.writeBlock(Line, Line.length());
        if(VRMLEnableDiffuseColor)
        {
          Line = "          diffuseColor ";
          // diffuse atom color
          if(atomBuffer->color != "")
          {
            Line += LineBuffer.setNum(VRMLDiffuseColorFactor * (float(atomBuffer->color.red()))/ 255, 'f') + " ";
            Line += LineBuffer.setNum(VRMLDiffuseColorFactor * (float(atomBuffer->color.green()))/ 255, 'f') + " ";
            Line += LineBuffer.setNum(VRMLDiffuseColorFactor * (float(atomBuffer->color.blue()))/ 255, 'f') + "\r\n";
          }
          else
            Line += " 1.0, 1.0, 1.0\r\n";
          VRMLFile.writeBlock(Line, Line.length());
        }
        if(VRMLEnableEmissiveColor)
        {
          Line = "          emissiveColor ";
          // emissive atom color
          if(atomBuffer->color != "")
          {
            Line += LineBuffer.setNum(VRMLEmissiveColorFactor * (float(atomBuffer->color.red()))/ 255, 'f') + " ";
            Line += LineBuffer.setNum(VRMLEmissiveColorFactor * (float(atomBuffer->color.green()))/ 255, 'f') + " ";
            Line += LineBuffer.setNum(VRMLEmissiveColorFactor * (float(atomBuffer->color.blue()))/ 255, 'f') + "\r\n";
          }
          else
            Line += " 0.0, 0.0, 0.0\r\n";
          VRMLFile.writeBlock(Line, Line.length());
        }
        if(VRMLEnableSpecularColor)
        {
          Line = "          specularColor ";
          // specular atom color
          if(atomBuffer->color != "")
          {
            Line += LineBuffer.setNum(VRMLSpecularColorFactor * (float(atomBuffer->color.red()))/ 255, 'f') + " ";
            Line += LineBuffer.setNum(VRMLSpecularColorFactor * (float(atomBuffer->color.green()))/ 255, 'f') + " ";
            Line += LineBuffer.setNum(VRMLSpecularColorFactor * (float(atomBuffer->color.blue()))/ 255, 'f') + "\r\n";
          }
          else
            Line += " 0.0, 0.0, 0.0\r\n";
          VRMLFile.writeBlock(Line, Line.length());
        }
        // shininess
        Line = "          shininess " + LineBuffer.setNum(VRMLShininess, 'f') + "\r\n";
        Line += "        }\r\n      }\r\n";
        VRMLFile.writeBlock(Line, Line.length());
        // geometry
        Line = "      geometry Sphere {\r\n        radius ";
        Line += LineBuffer.setNum(atomBuffer->radius, 'f');
        Line += "\r\n      }\r\n    }\r\n  ]\r\n";
        VRMLFile.writeBlock(Line, Line.length());
        // translation
        Line = "  translation ";
        Line += LineBuffer.setNum(atomBuffer->atomcoordinate.x, 'f') + " ";
        Line += LineBuffer.setNum(atomBuffer->atomcoordinate.y, 'f') + " ";
        Line += LineBuffer.setNum(atomBuffer->atomcoordinate.z, 'f') + "\r\n}\r\n";
				// end of Transform section
        VRMLFile.writeBlock(Line, Line.length());
      }
    }
    // write bonds
    if(NumberOfBonds > 0)
    {
      Line = "\r\n# BONDS\r\n";
      VRMLFile.writeBlock(Line, Line.length());
      VRMLBondList.first();
      for(i = 1; i <= NumberOfBonds; i++)
      {
        if(!(bondBuffer = (CVRMLBond*)VRMLBondList.current())) return FALSE;
        VRMLBondList.next();
        Line = "Transform {\r\n  children [\r\n    Shape {\r\n      appearance Appearance {\r\n";
        // texture mapping
        if(VRMLEnableTextureMapping)
        {
          Line += "        texture ImageTexture {\r\n";
          Line += "          url ";
          if (VRMLImageTextureURL.left(1) != "\"") Line += "\"";
          Line += VRMLImageTextureURL;
          if (VRMLImageTextureURL.right(1) != "\"") Line += "\"";
          Line += "\r\n        }\r\n";
        }
        // material
        Line += "        material Material {\r\n";
        VRMLFile.writeBlock(Line, Line.length());
        if(VRMLEnableDiffuseColor)
        {
          Line = "          diffuseColor ";
          // diffuse bond color
          if(bondBuffer->color != "")
          {
            Line += LineBuffer.setNum(VRMLDiffuseColorFactor * (float(bondBuffer->color.red()))/ 255, 'f') + " ";
            Line += LineBuffer.setNum(VRMLDiffuseColorFactor * (float(bondBuffer->color.green()))/ 255, 'f') + " ";
            Line += LineBuffer.setNum(VRMLDiffuseColorFactor * (float(bondBuffer->color.blue()))/ 255, 'f') + "\r\n";
          }
          else
            Line += " 1.0, 1.0, 1.0\r\n";
          VRMLFile.writeBlock(Line, Line.length());
        }
        if(VRMLEnableEmissiveColor)
        {
          Line = "          emissiveColor ";
          // emissive bond color
          if(bondBuffer->color != "")
          {
            Line += LineBuffer.setNum(VRMLEmissiveColorFactor * (float(bondBuffer->color.red()))/ 255, 'f') + " ";
            Line += LineBuffer.setNum(VRMLEmissiveColorFactor * (float(bondBuffer->color.green()))/ 255, 'f') + " ";
            Line += LineBuffer.setNum(VRMLEmissiveColorFactor * (float(bondBuffer->color.blue()))/ 255, 'f') + "\r\n";
          }
          else
            Line += " 0.0, 0.0, 0.0\r\n";
          VRMLFile.writeBlock(Line, Line.length());
        }
        if(VRMLEnableSpecularColor)
        {
          Line = "          specularColor ";
          // specular bond color
          if(bondBuffer->color != "")
          {
            Line += LineBuffer.setNum(VRMLSpecularColorFactor * (float(bondBuffer->color.red()))/ 255, 'f') + " ";
            Line += LineBuffer.setNum(VRMLSpecularColorFactor * (float(bondBuffer->color.green()))/ 255, 'f') + " ";
            Line += LineBuffer.setNum(VRMLSpecularColorFactor * (float(bondBuffer->color.blue()))/ 255, 'f') + "\r\n";
          }
          else
            Line += " 0.0, 0.0, 0.0\r\n";
          VRMLFile.writeBlock(Line, Line.length());
        }
        // shininess
        Line = "          shininess " + LineBuffer.setNum(VRMLShininess, 'f') + "\r\n";
        Line += "        }\r\n      }\r\n";
        VRMLFile.writeBlock(Line, Line.length());
        // calculate bond coordinates
        Atom1x = bondBuffer->bondcoordinate[1].GetX();
        Atom1y = bondBuffer->bondcoordinate[1].GetY();
        Atom1z = bondBuffer->bondcoordinate[1].GetZ();
        Atom2x = bondBuffer->bondcoordinate[2].GetX();
        Atom2y = bondBuffer->bondcoordinate[2].GetY();
        Atom2z = bondBuffer->bondcoordinate[2].GetZ();
        Bondx = Atom1x - Atom2x;
        Bondy = Atom1y - Atom2y;
        Bondz = Atom1z - Atom2z;
        BondLength = sqrt(Bondx * Bondx + Bondy * Bondy + Bondz * Bondz);
        // geometry
        Line = "      geometry Cylinder {\r\n        radius ";
        Line += LineBuffer.setNum(bondBuffer->radius, 'f') + "\r\n";
        Line += "        height " + LineBuffer.setNum(BondLength, 'f') + "\r\n";
        Line += "      }\r\n    }\r\n  ]\r\n";
        VRMLFile.writeBlock(Line, Line.length());
        // rotation
        if(BondLength > 0.0)
        {
          Line = "  rotation ";
          Line += LineBuffer.setNum(Bondz, 'f') + " ";
          Line += LineBuffer.setNum(0.0, 'f') + " ";
          Line += LineBuffer.setNum(-Bondx, 'f') + " ";
          Line += LineBuffer.setNum(acos(Bondy / BondLength), 'f') + "\r\n";
          VRMLFile.writeBlock(Line, Line.length());
        }
        // translation
        Line = "  translation ";
        Line += LineBuffer.setNum(Atom1x - 0.5 * Bondx, 'f') + " ";
        Line += LineBuffer.setNum(Atom1y - 0.5 * Bondy, 'f') + " ";
        Line += LineBuffer.setNum(Atom1z - 0.5 * Bondz, 'f') + "\r\n}\r\n";
				// end of Transform section
        VRMLFile.writeBlock(Line, Line.length());
      }
    }
    // write foot
    if(VRMLFoot != "")
    {
      Line = "# " + VRMLFoot + "\r\n\r\n";
      VRMLFile.writeBlock(Line, Line.length());
    }
    // end of file
    VRMLFile.close();
    return(TRUE);
  }
  else
  {
    // open file in write-only mode fails
    return(FALSE);
  }
}

/////////////////////////////////////////////////////////////////////////////
// SetAtom
int CVRMLFile::SetAtom(CVRMLAtom* NewAtom)
{
  if(!(atom = new CVRMLAtom))
  {
    Reset();
    return (FALSE);
  }
  NumberOfAtoms++;
  *atom = *NewAtom;
  VRMLAtomList.append(atom);
  return(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// SetBond
int CVRMLFile::SetBond(CVRMLBond* NewBond)
{
  if(!(bond = new CVRMLBond))
  {
    Reset();
    return (FALSE);
  }
  NumberOfBonds++;
  *bond = *NewBond;
  VRMLBondList.append(bond);
  return(TRUE);
}  
