/***************************************************************************
                          xyzfile.cpp  -  description
                             -------------------
    begin                : Thu Feb 3 2000
    copyright            : (C) 2000 - 2003 by M. Hoverath
    email                : mario.hoverath@uni-koeln.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files
#include <xyzfile.h>


/////////////////////////////////////////////////////////////////////////////
// definition of the class CXYZCoordinate
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Constructor
CXYZCoordinate::CXYZCoordinate(QObject *parent, const char *name)
  : QObject(parent, name)
{
  Atomsymbol = "";
  x = (float)0.0;
  y = (float)0.0;
  z = (float)0.0;
}

/////////////////////////////////////////////////////////////////////////////
// destructor
CXYZCoordinate::~CXYZCoordinate()
{

}

/////////////////////////////////////////////////////////////////////////////
// = operator
CXYZCoordinate& CXYZCoordinate::operator=(CXYZCoordinate &XYZAtomcoordinate)
{
  // equalize members
  Atomsymbol = XYZAtomcoordinate.Atomsymbol;
  x = XYZAtomcoordinate.x;
  y = XYZAtomcoordinate.y;
  z = XYZAtomcoordinate.z;

  return *this;
}

/////////////////////////////////////////////////////////////////////////////
// Serialize
void CXYZCoordinate::Serialize()
{
  // not yet implemented
}

/////////////////////////////////////////////////////////////////////////////
// definition of the class CXYZFile
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Constructor
CXYZFile::CXYZFile(QObject *parent, const char *name)
  : QObject(parent, name)
{
  // enabling auto-delete will delete objects that are removed
  XYZAtomcoordinateList.setAutoDelete(TRUE);
  Reset();
}

/////////////////////////////////////////////////////////////////////////////
// destructor
CXYZFile::~CXYZFile()
{  
  // auto-delete is enabled!
  XYZAtomcoordinateList.clear();
}

/////////////////////////////////////////////////////////////////////////////
// = operator
CXYZFile& CXYZFile::operator=(CXYZFile &XYZFile)
{
  Reset();
  Title = XYZFile.Title;
  NumberOfAtoms = XYZFile.NumberOfAtoms;

  // XYZAtomcoordinate-List
  XYZAtomcoordinateList.clear();
  CXYZCoordinate coordinateBuffer;
  if(XYZFile.XYZAtomcoordinateList.first())
  {
    for(uint i = 0; i <= XYZFile.XYZAtomcoordinateList.count() - 1; i++)
    {
      coordinateBuffer = *(CXYZCoordinate*)XYZFile.XYZAtomcoordinateList.current();
      coordinate = new CXYZCoordinate;
      if(!coordinate)
      {
        Reset();
        return *this;
      }
      *coordinate = coordinateBuffer;
      XYZAtomcoordinateList.append(coordinate);
      XYZFile.XYZAtomcoordinateList.next();
    }
  }
  return *this;
}

/////////////////////////////////////////////////////////////////////////////
// Serialize
void CXYZFile::Serialize(void)
{
  // not yet implemented
}

/////////////////////////////////////////////////////////////////////////////
// Reset
void CXYZFile::Reset(void)
{  
  XYZAtomcoordinateList.clear();
  NumberOfAtoms = 0;
  Title = "";
}

/////////////////////////////////////////////////////////////////////////////
// LoadFile
int CXYZFile::LoadFile(QString Path)
{
  // initialisations
  Reset();
  QFile XYZFile;
  int LineNumber = 0;
  QString Line = "";
  int tmpNumberOfAtoms = 0;
  int emptyLineFollows = 0;

  XYZFile.setName(Path);
  if(!XYZFile.open(IO_ReadOnly))
  {
    return FALSE;
  }
  else
  {
    while(!XYZFile.atEnd() && !emptyLineFollows)
    {
      if(XYZFile.readLine(Line, XYZ_FILE_MAX_LINE_LENGTH) == -1)
      {
        Reset();
        XYZFile.close();
        return FALSE;
      }
      LineNumber++;
      Line = Line.simplifyWhiteSpace();
      switch (LineNumber)
      {
        case 1:
        {
          // declared number of atoms
          tmpNumberOfAtoms = Line.toInt();
          if(tmpNumberOfAtoms <= 0)
          {
            Reset();
            XYZFile.close();
            return FALSE;
          }          
          break;
        }
        case 2:
        {
          // the title
          Title = Line;
          break;
        }
        default:
        {
          // the structure
          // check if the line is empty
          if(!Line.isEmpty())
          {
            if(!(coordinate = new CXYZCoordinate))
            {
              Reset();
              XYZFile.close();
              return FALSE;
            }
            NumberOfAtoms++;
            if(Line.find(" ") != -1)
            {
              coordinate->Atomsymbol = Line.left(Line.find(" "));
              Line = Line.right(Line.length() - Line.find(" ") - 1);
              if(Line.find(" ") != -1)
              {
                coordinate->x = (Line.left(Line.find(" "))).toFloat();
                Line = Line.right(Line.length() - Line.find(" ") - 1);
                if(Line.find(" ") != -1)
                {
                  coordinate->y = (Line.left(Line.find(" "))).toFloat();
                  Line = Line.right(Line.length() - Line.find(" ") - 1);
                  if(!Line.isEmpty())
                  {
                    coordinate->z = Line.toFloat();
                    XYZAtomcoordinateList.append(coordinate);
                  }
                  else
                  {
                    Reset();
                    XYZFile.close();
                    return FALSE;
                  }      
                }
                else
                {
                  Reset();
                  XYZFile.close();
                  return FALSE;
                }      
              }
              else
              {
                Reset();
                XYZFile.close();
                return FALSE;
              }      
            }
            else
            {
              Reset();
              XYZFile.close();
              return FALSE;
            }      
          }
          else
          {
            // stop reading the file
            emptyLineFollows = 1;
          }
        }
      }
    }
    if(tmpNumberOfAtoms != NumberOfAtoms)
    {
      Reset();
      XYZFile.close();
      return FALSE;
    }
  }
  XYZFile.close();
  return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// SaveFile
int CXYZFile::SaveFile(QString Path)
{
  // Initialisations
  CXYZCoordinate* coordinateBuffer;
  QString LineBuffer = "";
  int i = 0;
  QFile XYZFile;
  QString Line = "";

  XYZFile.setName(Path);
  if (XYZFile.open(IO_WriteOnly))
  {
    // number of atoms
    Line = Line.setNum(NumberOfAtoms) + "\r\n";  
    XYZFile.writeBlock(Line, Line.length());
    // title
    Line = Title + "\r\n";
    XYZFile.writeBlock(Line, Line.length());
    // structure
    XYZAtomcoordinateList.first();
    for(i = 1; i <= NumberOfAtoms; i++)
    {
      if(!(coordinateBuffer = (CXYZCoordinate*)XYZAtomcoordinateList.current())) return FALSE;
      XYZAtomcoordinateList.next();
      Line.fill(' ', 6 - (coordinateBuffer->Atomsymbol).length());
      Line = coordinateBuffer->Atomsymbol + Line;
      if(coordinateBuffer->x > 0) Line += " ";
      Line += LineBuffer.setNum(coordinateBuffer->x, 'f') + "  ";
      if(coordinateBuffer->y > 0) Line += " ";
      Line += LineBuffer.setNum(coordinateBuffer->y, 'f') + "  ";
      if(coordinateBuffer->z > 0) Line += " ";
      Line += LineBuffer.setNum(coordinateBuffer->z, 'f') + "\r\n";
      XYZFile.writeBlock(Line, Line.length());
    }
    XYZFile.close();
    return TRUE;
  }
  else
    return FALSE;
}

/////////////////////////////////////////////////////////////////////////////
// Get...-Methods

/////////////////////////////////////////////////////////////////////////////
// GetNumberOfAtoms
int CXYZFile::GetNumberOfAtoms(void)
{
  return NumberOfAtoms;
}

/////////////////////////////////////////////////////////////////////////////
// GetTitle
QString CXYZFile::GetTitle(void)
{
  return Title;
}

/////////////////////////////////////////////////////////////////////////////
// GetCoordinate
int CXYZFile::GetCoordinate(int LineNumber, CXYZCoordinate& Coordinate)
{
  if(LineNumber > GetNumberOfAtoms()) return FALSE;
  CXYZCoordinate* coordinateBuffer = NULL;
  int i = 0;
  XYZAtomcoordinateList.first();
  for(i = 1; i <= LineNumber; i++)
  {
    if(!(coordinateBuffer = (CXYZCoordinate*)XYZAtomcoordinateList.current())) return FALSE;
    XYZAtomcoordinateList.next();
  }
  Coordinate = *coordinateBuffer;
  return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// Set...-Methods

/////////////////////////////////////////////////////////////////////////////
// SetTitle
int CXYZFile::SetTitle(QString title)
{
  Title = title;
  return 0;
}

/////////////////////////////////////////////////////////////////////////////
// SetCoordinate
int CXYZFile::SetCoordinate(CXYZCoordinate& Coordinate)
{
  if(!(coordinate = new CXYZCoordinate))
  {
    Reset();
    return FALSE;
  }
  NumberOfAtoms++;
  *coordinate = Coordinate;
  coordinate->Atomsymbol = (coordinate->Atomsymbol).stripWhiteSpace();
  XYZAtomcoordinateList.append(coordinate);
  return TRUE;
}
